@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/result.c,v 1.64.2.2 2002/04/18 15:19:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/result.c,v 1.84.2.2 2003/09/18 15:43:32 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -27,6 +27,14 @@
  *
  *  result.c - wait for an ldap result
  */
+/* Note: A verbatim copy of version 2.0.1 of the OpenLDAP Public License 
+ * can be found in the file "build/LICENSE-2.0.1" in this distribution
+ * of OpenLDAP Software.
+ */
+/*
+ * Portions Copyright (C) The Internet Society (1997)
+ * ASN.1 fragments are from RFC 2251; see RFC for full legal notices.
+ */
 
 /*
  * LDAPv3 (RFC2251)
@@ -53,7 +61,7 @@
 #include <ac/unistd.h>
 
 #include "ldap-int.h"
-
+#include "ldap_log.h"
 
 static int ldap_abandoned LDAP_P(( LDAP *ld, ber_int_t msgid ));
 static int ldap_mark_abandoned LDAP_P(( LDAP *ld, ber_int_t msgid ));
@@ -93,34 +101,33 @@ ldap_result(
 	LDAPMessage **result )
 {
 	LDAPMessage	*lm;
+	int	rc;
 
 	assert( ld != NULL );
 	assert( result != NULL );
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_ARGS, "ldap_result msgid %d\n", msgid ));
+	LDAP_LOG ( OPERATION, ARGS, "ldap_result msgid %d\n", msgid, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_result msgid %d\n", msgid, 0, 0 );
 #endif
 
-	if( ld == NULL ) {
-		return -1;
-	}
-
-	if( result == NULL ) {
-		ld->ld_errno = LDAP_PARAM_ERROR;
-		return -1;
-	}
-
-    lm = chkResponseList(ld, msgid, all);
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_res_mutex );
+#endif
+	lm = chkResponseList(ld, msgid, all);
 
 	if ( lm == NULL ) {
-		return( wait4msg( ld, msgid, all, timeout, result ) );
+		rc = wait4msg( ld, msgid, all, timeout, result );
+	} else {
+		*result = lm;
+		ld->ld_errno = LDAP_SUCCESS;
+		rc = lm->lm_msgtype;
 	}
-
-	*result = lm;
-	ld->ld_errno = LDAP_SUCCESS;
-	return( lm->lm_msgtype );
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
+#endif
+	return( rc );
 }
 
 static LDAPMessage *
@@ -138,8 +145,8 @@ chkResponseList(
 	 */
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_ARGS, 
-		"ldap_chkResponseList for msgid=%d, all=%d\n", msgid, all ));
+	LDAP_LOG ( OPERATION, ARGS, "ldap_chkResponseList for msgid=%d, all=%d\n", 
+		msgid, all, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"ldap_chkResponseList for msgid=%d, all=%d\n",
@@ -151,8 +158,8 @@ chkResponseList(
 
 		if ( ldap_abandoned( ld, lm->lm_msgid ) ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
-				"ldap_chkResponseList msg abandoned, msgid %d\n", msgid ));
+			LDAP_LOG ( OPERATION, DETAIL1, 
+				"ldap_chkResponseList msg abandoned, msgid %d\n", msgid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"ldap_chkResponseList msg abandoned, msgid %d\n",
@@ -182,7 +189,7 @@ chkResponseList(
 			for ( tmp = lm; tmp != NULL; tmp = tmp->lm_chain ) {
 				if ( tmp->lm_msgtype != LDAP_RES_SEARCH_ENTRY
 				    && tmp->lm_msgtype != LDAP_RES_SEARCH_REFERENCE
-					&& tmp->lm_msgtype != LDAP_RES_EXTENDED_PARTIAL )
+					&& tmp->lm_msgtype != LDAP_RES_INTERMEDIATE )
 				{
 					break;
 				}
@@ -216,17 +223,17 @@ chkResponseList(
 #ifdef LDAP_DEBUG
 	if( lm == NULL) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_RESULTS, 
-			"ldap_chkResponseList returns NULL\n" ));
+		LDAP_LOG ( OPERATION, RESULTS, "ldap_chkResponseList returns NULL\n",
+			0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_chkResponseList returns NULL\n", 0, 0, 0);
 #endif
 	} else {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_RESULTS, 
-			"ldap_chkResponseList returns msgid %d, type 0x02lu\n",
-			lm->lm_msgid, (unsigned long) lm->lm_msgtype ));
+		LDAP_LOG ( OPERATION, RESULTS, 
+			"ldap_chkResponseList returns msgid %d, type 0x%02lu\n",
+			lm->lm_msgid, (unsigned long) lm->lm_msgtype, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_chkResponseList returns msgid %d, type 0x%02lu\n",
@@ -257,17 +264,17 @@ wait4msg(
 #ifdef LDAP_DEBUG
 	if ( timeout == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_ARGS,
-			"wait4msg (infinite timeout), msgid %d\n", msgid ));
+		LDAP_LOG ( OPERATION, ARGS, 
+			"wait4msg (infinite timeout), msgid %d\n", msgid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE, "wait4msg (infinite timeout), msgid %d\n",
 		    msgid, 0, 0 );
 #endif
 	} else {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_ARGS,
+		LDAP_LOG ( OPERATION, ARGS, 
 			"wait4msg (timeout %ld sec, %ld usec), msgid %d\n", 
-			(long) timeout->tv_sec, (long) timeout->tv_usec, msgid ));
+			(long) timeout->tv_sec, (long) timeout->tv_usec, msgid );
 #else
 		Debug( LDAP_DEBUG_TRACE, "wait4msg (timeout %ld sec, %ld usec), msgid %d\n",
 		       (long) timeout->tv_sec, (long) timeout->tv_usec, msgid );
@@ -287,8 +294,8 @@ wait4msg(
 	while ( rc == -2 ) {
 #ifdef LDAP_DEBUG
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_ARGS,
-			"wait4msg continue, msgid %d, all %d\n", msgid, all ));
+		LDAP_LOG ( OPERATION, ARGS, 
+			"wait4msg continue, msgid %d, all %d\n", msgid, all, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE, "wait4msg continue, msgid %d, all %d\n",
 		    msgid, all, 0 );
@@ -314,14 +321,12 @@ wait4msg(
 
 		    if ( lc == NULL ) {
 			    rc = ldap_int_select( ld, tvp );
-
-
 #ifdef LDAP_DEBUG
 			    if ( rc == -1 ) {
 #ifdef NEW_LOGGING
-					LDAP_LOG (( "result", LDAP_LEVEL_ARGS,
+					LDAP_LOG ( OPERATION, ARGS, 
 						"wait4msg: ldap_int_select returned -1: errno %d\n", 
-						errno ));
+						errno, 0, 0 );
 #else
 			        Debug( LDAP_DEBUG_TRACE,
 				        "ldap_int_select returned -1: errno %d\n",
@@ -343,6 +348,18 @@ wait4msg(
 				    rc = -2;	/* select interrupted: loop */
 			    } else {
 				    rc = -2;
+#ifdef LDAP_R_COMPILE
+				    ldap_pvt_thread_mutex_lock( &ld->ld_req_mutex );
+#endif
+				    if ( ld->ld_requests &&
+						ld->ld_requests->lr_status == LDAP_REQST_WRITING &&
+						ldap_is_write_ready( ld,
+							ld->ld_requests->lr_conn->lconn_sb ) ) {
+						ldap_int_flush_request( ld, ld->ld_requests );
+					}
+#ifdef LDAP_R_COMPILE
+				    ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
+#endif
 				    for ( lc = ld->ld_conns; rc == -2 && lc != NULL;
 				        lc = nextlc ) {
 					    nextlc = lc->lconn_next;
@@ -367,8 +384,8 @@ wait4msg(
 			}
 
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1,
-				"wait4msg: %ld secs to go\n", (long) tv.tv_sec ));
+			LDAP_LOG ( OPERATION, DETAIL1, 
+				"wait4msg: %ld secs to go\n", (long) tv.tv_sec, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE, "wait4msg:  %ld secs to go\n",
 			       (long) tv.tv_sec, 0, 0 );
@@ -400,6 +417,9 @@ try_read1msg(
 	BerElement	tmpber;
 	int		rc, refer_cnt, hadref, simple_request;
 	ber_int_t	lderr;
+#ifdef LDAP_CONNECTIONLESS
+	int		firstmsg = 1, moremsgs = 0, isv2 = 0;
+#endif
 	/*
 	 * v3ref = flag for V3 referral / search reference
 	 * 0 = not a ref, 1 = sucessfully chased ref, -1 = pass ref to application
@@ -410,19 +430,19 @@ try_read1msg(
 	assert( lc != NULL );
 	
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_ARGS, "read1msg: msgid %d, all %d\n",
-		msgid, all ));
+	LDAP_LOG ( OPERATION, ARGS, "read1msg: msgid %d, all %d\n", msgid, all, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "read1msg: msgid %d, all %d\n", msgid, all, 0 );
 #endif
 
-    if ( lc->lconn_ber == NULL ) {
+retry:
+	if ( lc->lconn_ber == NULL ) {
 		lc->lconn_ber = ldap_alloc_ber_with_options(ld);
 
 		if( lc->lconn_ber == NULL ) {
 			return -1;
 		}
-    }
+	}
 
 	ber = lc->lconn_ber;
 	assert( LBER_VALID (ber) );
@@ -433,15 +453,24 @@ try_read1msg(
 	if ( LDAP_IS_UDP(ld) ) {
 		struct sockaddr from;
 		ber_int_sb_read(sb, &from, sizeof(struct sockaddr));
+		if (ld->ld_options.ldo_version == LDAP_VERSION2) isv2=1;
 	}
+nextresp3:
 #endif
-	if ( (tag = ber_get_next( sb, &len, ber ))
-	    != LDAP_TAG_MESSAGE ) {
+	tag = ber_get_next( sb, &len, ber );
+	if ( tag == LDAP_TAG_MESSAGE ) {
+		/*
+	 	 * We read a complete message.
+	 	 * The connection should no longer need this ber.
+	 	 */
+		lc->lconn_ber = NULL;
+	}
+	if ( tag != LDAP_TAG_MESSAGE ) {
 		if ( tag == LBER_DEFAULT) {
 #ifdef LDAP_DEBUG		   
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
-				"read1msg: ber_get_next failed\n" ));
+			LDAP_LOG ( OPERATION, DETAIL1, 
+				"read1msg: ber_get_next failed\n", 0, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_CONNS,
 			      "ber_get_next failed.\n", 0, 0, 0 );
@@ -460,12 +489,6 @@ try_read1msg(
 		return -1;
 	}
 
-	/*
-     * We read a complete message.
-	 * The connection should no longer need this ber.
-	 */
-    lc->lconn_ber = NULL;
-
 	/* message id */
 	if ( ber_get_int( ber, &id ) == LBER_ERROR ) {
 		ber_free( ber, 1 );
@@ -475,34 +498,36 @@ try_read1msg(
 
 	/* if it's been abandoned, toss it */
 	if ( ldap_abandoned( ld, id ) ) {
-		ber_free( ber, 1 );
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
-			"read1msg: abandoned\n" ));
+		LDAP_LOG ( OPERATION, DETAIL1, "read1msg: abandoned\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "abandoned\n", 0, 0, 0);
 #endif
+retry_ber:
+		ber_free( ber, 1 );
+		if ( ber_sockbuf_ctrl( sb, LBER_SB_OPT_DATA_READY, NULL ) ) {
+			goto retry;
+		}
 		return( -2 );	/* continue looking */
 	}
 
 	if (( lr = ldap_find_request_by_msgid( ld, id )) == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+		LDAP_LOG ( OPERATION, DETAIL1, 
 			"read1msg: no request for response with msgid %ld (tossing)\n",
-			(long) id ));
+			(long) id, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "no request for response with msgid %ld (tossing)\n",
 		    (long) id, 0, 0 );
 #endif
-		ber_free( ber, 1 );
-		return( -2 );	/* continue looking */
+		goto retry_ber;
 	}
 #ifdef LDAP_CONNECTIONLESS
-	if (LDAP_IS_UDP(ld) && ld->ld_options.ldo_version == LDAP_VERSION2) {
-		struct berval blank;
-		ber_scanf(ber, "m{", &blank);
+	if (LDAP_IS_UDP(ld) && isv2) {
+		ber_scanf(ber, "x{");
 	}
+nextresp2:
 #endif
 	/* the message type */
 	if ( (tag = ber_peek_tag( ber, &len )) == LBER_ERROR ) {
@@ -512,10 +537,10 @@ try_read1msg(
 	}
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+	LDAP_LOG ( OPERATION, DETAIL1, 
 		"read1msg: ldap_read: message type %s msgid %ld, original id %ld\n",
 	    ldap_int_msgtype2str( tag ),
-		(long) lr->lr_msgid, (long) lr->lr_origid ));
+		(long) lr->lr_msgid, (long) lr->lr_origid );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"ldap_read: message type %s msgid %ld, original id %ld\n",
@@ -557,10 +582,10 @@ try_read1msg(
 						if( lr->lr_status != LDAP_REQST_COMPLETED) {
 							lr->lr_status = LDAP_REQST_CHASINGREFS;
 #ifdef NEW_LOGGING
-							LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+							LDAP_LOG ( OPERATION, DETAIL1, 
 								"read1msg: search ref chased,"
 								"mark request chasing refs, id =	%d\n",
-								lr->lr_msgid ));
+								lr->lr_msgid, 0, 0 );
 #else
 							Debug( LDAP_DEBUG_TRACE,
 							    "read1msg:  search ref chased, mark request chasing refs, id = %d\n",
@@ -590,10 +615,10 @@ try_read1msg(
 							rc = LDAP_DECODING_ERROR;
 							lr->lr_status = LDAP_REQST_COMPLETED;
 #ifdef NEW_LOGGING
-							LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+							LDAP_LOG ( OPERATION, DETAIL1, 
 								"read1msg: referral decode error,"
 								"mark request completed, id =	%d\n",
-								lr->lr_msgid ));
+								lr->lr_msgid, 0, 0 );
 #else
 							Debug( LDAP_DEBUG_TRACE,
 							    "read1msg: referral decode error, mark request completed, id = %d\n",
@@ -607,10 +632,10 @@ try_read1msg(
 							    0, &lr->lr_res_error, &hadref );
 							lr->lr_status = LDAP_REQST_COMPLETED;
 #ifdef NEW_LOGGING
-							LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+							LDAP_LOG ( OPERATION, DETAIL1, 
 								"read1msg: referral chased,"
 								"mark request completed, id =	%d\n",
-								lr->lr_msgid ));
+								lr->lr_msgid, 0, 0 );
 #else
 							Debug( LDAP_DEBUG_TRACE,
 							    "read1msg:  referral chased, mark request completed, id = %d\n",
@@ -639,7 +664,9 @@ try_read1msg(
 	 * go through the following code.  This code also chases V2 referrals
 	 * and checks if all referrals have been chased.
 	 */
-	if ( (tag != LDAP_RES_SEARCH_ENTRY) && (v3ref > -1) ) {
+	if ( (tag != LDAP_RES_SEARCH_ENTRY) && (v3ref > -1) &&
+		(tag != LDAP_RES_INTERMEDIATE ))
+	{
 		/* For a v3 search referral/reference, only come here if already chased it */
 		if ( ld->ld_version >= LDAP_VERSION2 &&
 			( lr->lr_parent != NULL ||
@@ -647,8 +674,13 @@ try_read1msg(
 		{
 			tmpber = *ber;	/* struct copy */
 			if ( v3ref == 1 ) {
-				/* V3 search reference or V3 referral successfully chased */
-				refer_cnt = 0;
+				/* V3 search reference or V3 referral
+				 * sucessfully chased. If this message
+				 * is a search result, then it has no more
+				 * outstanding referrals.
+				 */
+				if ( tag == LDAP_RES_SEARCH_RESULT )
+					refer_cnt = 0;
 			} else if ( ber_scanf( &tmpber, "{iaa}", &lderr,
 			    &lr->lr_res_matched, &lr->lr_res_error )
 			    != LBER_ERROR ) {
@@ -658,10 +690,10 @@ try_read1msg(
 						&lr->lr_res_error, -1, &hadref );
 					lr->lr_status = LDAP_REQST_COMPLETED;
 #ifdef NEW_LOGGING
-					LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+					LDAP_LOG ( OPERATION, DETAIL1, 
 						"read1msg: V2 referral chased,"
 						"mark request completed, id =	%d\n",
-						lr->lr_msgid ));
+						lr->lr_msgid, 0, 0 );
 #else
 					Debug( LDAP_DEBUG_TRACE,
 					    "read1msg:  V2 referral chased, mark request completed, id = %d\n", lr->lr_msgid, 0, 0);
@@ -679,10 +711,10 @@ try_read1msg(
 					lr->lr_res_errno = LDAP_PARTIAL_RESULTS;
 				}
 #ifdef NEW_LOGGING
-LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+LDAP_LOG ( OPERATION, DETAIL1, 
 	"read1msg: new result: res_errno: %d, res_error: <%s>, res_matched: <%s>\n",
     lr->lr_res_errno, lr->lr_res_error ? lr->lr_res_error : "",
-    lr->lr_res_matched ? lr->lr_res_matched : "" ));
+    lr->lr_res_matched ? lr->lr_res_matched : "" );
 #else
 Debug( LDAP_DEBUG_TRACE,
     "new result:  res_errno: %d, res_error: <%s>, res_matched: <%s>\n",
@@ -693,8 +725,8 @@ Debug( LDAP_DEBUG_TRACE,
 		}
 
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
-			"read1msg: %d new referrals\n", refer_cnt ));
+		LDAP_LOG ( OPERATION, DETAIL1, "read1msg: %d new referrals\n", 
+			refer_cnt, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 		    "read1msg:  %d new referrals\n", refer_cnt, 0, 0 );
@@ -719,8 +751,9 @@ Debug( LDAP_DEBUG_TRACE,
 
 			lr->lr_status = LDAP_REQST_COMPLETED; /* declare this request done */
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
-				"read1msg: mark request completed, id = %d\n", lr->lr_msgid ));
+			LDAP_LOG ( OPERATION, DETAIL1, 
+				"read1msg: mark request completed, id = %d\n", 
+				lr->lr_msgid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 			    "read1msg:  mark request completed, id = %d\n", lr->lr_msgid, 0, 0);
@@ -749,12 +782,12 @@ Debug( LDAP_DEBUG_TRACE,
 				id = lr->lr_msgid;
 				tag = lr->lr_res_msgtype;
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
-				"read1msg: request %ld done\n", (long) id ));
-			LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+			LDAP_LOG ( OPERATION, DETAIL1, 
+				"read1msg: request %ld done\n", (long) id, 0, 0 );
+			LDAP_LOG ( OPERATION, DETAIL1, 
 				"read1msg: res_errno: %d,res_error: <%s>, res_matched: <%s>\n",
 				lr->lr_res_errno, lr->lr_res_error ? lr->lr_res_error : "",
-				lr->lr_res_matched ? lr->lr_res_matched : "" ));
+				lr->lr_res_matched ? lr->lr_res_matched : "" );
 #else
 				Debug( LDAP_DEBUG_ANY, "request %ld done\n",
 				    (long) id, 0, 0 );
@@ -795,11 +828,84 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 	new->lm_msgtype = tag;
 	new->lm_ber = ber;
 
-#ifndef LDAP_NOCACHE
-		if ( ld->ld_cache != NULL ) {
-			ldap_add_result_to_cache( ld, new );
+#ifdef LDAP_CONNECTIONLESS
+	/* CLDAP replies all fit in a single datagram. In LDAPv2 RFC1798
+	 * the responses are all a sequence wrapped in one message. In
+	 * LDAPv3 each response is in its own message. The datagram must
+	 * end with a SearchResult. We can't just parse each response in
+	 * separate calls to try_read1msg because the header info is only
+	 * present at the beginning of the datagram, not at the beginning
+	 * of each response. So parse all the responses at once and queue
+	 * them up, then pull off the first response to return to the
+	 * caller when all parsing is complete.
+	 */
+	if ( LDAP_IS_UDP(ld) ) {
+		/* If not a result, look for more */
+		if ( tag != LDAP_RES_SEARCH_RESULT ) {
+			int ok = 0;
+			moremsgs = 1;
+			if (isv2) {
+				/* LDAPv2: dup the current ber, skip past the current
+				 * response, and see if there are any more after it.
+				 */
+				ber = ber_dup( ber );
+				ber_scanf( ber, "x" );
+				if (ber_peek_tag(ber, &len) != LBER_DEFAULT) {
+					/* There's more - dup the ber buffer so they can all be
+					 * individually freed by ldap_msgfree.
+					 */
+					struct berval bv;
+					ber_get_option(ber, LBER_OPT_BER_REMAINING_BYTES, &len);
+					bv.bv_val = LDAP_MALLOC(len);
+					if (bv.bv_val) {
+						ok=1;
+						ber_read(ber, bv.bv_val, len);
+						bv.bv_len = len;
+						ber_init2(ber, &bv, ld->ld_lberoptions );
+					}
+				}
+			} else {
+				/* LDAPv3: Just allocate a new ber. Since this is a buffered
+				 * datagram, if the sockbuf is readable we still have data
+				 * to parse.
+				 */
+				ber = ldap_alloc_ber_with_options(ld);
+				if (ber_sockbuf_ctrl(sb, LBER_SB_OPT_DATA_READY, NULL)) ok=1;
+			}
+			/* set up response chain */
+			if ( firstmsg ) {
+				firstmsg = 0;
+				new->lm_next = ld->ld_responses;
+				ld->ld_responses = new;
+			} else {
+				tmp->lm_chain = new;
+			}
+			tmp = new;
+			/* "ok" means there's more to parse */
+			if (ok) {
+				if (isv2) goto nextresp2;
+				else goto nextresp3;
+			} else {
+				/* got to end of datagram without a SearchResult. Free
+				 * our dup'd ber, but leave any buffer alone. For v2 case,
+				 * the previous response is still using this buffer. For v3,
+				 * the new ber has no buffer to free yet.
+				 */
+				ber_free(ber, 0);
+				return -1;
+			}
+		} else if ( moremsgs ) {
+		/* got search result, and we had multiple responses in 1 datagram.
+		 * stick the result onto the end of the chain, and then pull the
+		 * first response off the head of the chain.
+		 */
+			tmp->lm_chain = new;
+			*result = chkResponseList( ld, msgid, all );
+			ld->ld_errno = LDAP_SUCCESS;
+			return( (*result)->lm_msgtype );
 		}
-#endif /* LDAP_NOCACHE */
+	}
+#endif
 
 	/* is this the one we're looking for? */
 	if ( msgid == LDAP_RES_ANY || id == msgid ) {
@@ -832,19 +938,18 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 	if ( l == NULL ) {
 		if ( foundit ) {
 			*result = new;
-			ld->ld_errno = LDAP_SUCCESS;
-			return( tag );
+			goto exit;
 		}
 
 		new->lm_next = ld->ld_responses;
 		ld->ld_responses = new;
-		return( -2 );	/* continue looking */
+		goto exit;
 	}
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
+	LDAP_LOG ( OPERATION, DETAIL1, 
 		"read1msg: adding response id %ld type %ld\n",
-		(long) new->lm_msgid, (long) new->lm_msgtype ));
+		(long) new->lm_msgid, (long) new->lm_msgtype, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "adding response id %ld type %ld:\n",
 	    (long) new->lm_msgid, (long) new->lm_msgtype, 0 );
@@ -854,7 +959,7 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 	for ( tmp = l; (tmp->lm_chain != NULL) &&
 	    	((tmp->lm_chain->lm_msgtype == LDAP_RES_SEARCH_ENTRY) ||
 	    	 (tmp->lm_chain->lm_msgtype == LDAP_RES_SEARCH_REFERENCE) ||
-			 (tmp->lm_chain->lm_msgtype == LDAP_RES_EXTENDED_PARTIAL ));
+			 (tmp->lm_chain->lm_msgtype == LDAP_RES_INTERMEDIATE ));
 	    tmp = tmp->lm_chain )
 		;	/* NULL */
 	tmp->lm_chain = new;
@@ -866,21 +971,16 @@ lr->lr_res_matched ? lr->lr_res_matched : "" );
 		else
 			prev->lm_next = l->lm_next;
 		*result = l;
-		ld->ld_errno = LDAP_SUCCESS;
-#ifdef LDAP_WORLD_P16
-		/*
-		 * XXX questionable fix; see text for [P16] on
-		 * http://www.critical-angle.com/ldapworld/patch/
-		 *
-		 * inclusion of this patch causes searchs to hang on
-		 * multiple platforms
-		 */
-		return( l->lm_msgtype );
-#else	/* LDAP_WORLD_P16 */
-		return( tag );
-#endif	/* !LDAP_WORLD_P16 */
 	}
 
+exit:
+	if ( foundit ) {
+		ld->ld_errno = LDAP_SUCCESS;
+		return( tag );
+	}
+	if ( ber_sockbuf_ctrl( sb, LBER_SB_OPT_DATA_READY, NULL ) ) {
+		goto retry;
+	}
 	return( -2 );	/* continue looking */
 }
 
@@ -968,12 +1068,12 @@ merge_error_info( LDAP *ld, LDAPRequest *parentr, LDAPRequest *lr )
 	}
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_DETAIL1, 
-		"read1msg: merged parent (id %d) error info: result errno %d, "
-		"error <%s>, matched <%s>\n", parentr->lr_msgid,
+	LDAP_LOG( OPERATION, DETAIL1, "merged parent (id %d) error info:  ",
+	    parentr->lr_msgid, 0, 0 );
+	LDAP_LOG( OPERATION, DETAIL1, "result errno %d, error <%s>, matched <%s>\n",
 	    parentr->lr_res_errno, parentr->lr_res_error ?
 	    parentr->lr_res_error : "", parentr->lr_res_matched ?
-	    parentr->lr_res_matched : "" ));
+	    parentr->lr_res_matched : "" );
 #else
 	Debug( LDAP_DEBUG_TRACE, "merged parent (id %d) error info:  ",
 	    parentr->lr_msgid, 0, 0 );
@@ -1011,7 +1111,7 @@ char * ldap_int_msgtype2str( ber_tag_t tag )
 	case LDAP_RES_COMPARE: return "compare";
 	case LDAP_RES_DELETE: return "delete";
 	case LDAP_RES_EXTENDED: return "extended-result";
-	case LDAP_RES_EXTENDED_PARTIAL: return "extended-partial";
+	case LDAP_RES_INTERMEDIATE: return "intermediate";
 	case LDAP_RES_MODIFY: return "modify";
 	case LDAP_RES_RENAME: return "rename";
 	case LDAP_RES_SEARCH_ENTRY: return "search-entry";
@@ -1028,7 +1128,7 @@ ldap_msgfree( LDAPMessage *lm )
 	int		type = 0;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_ENTRY, "ldap_msgfree\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_msgfree\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_msgfree\n", 0, 0, 0 );
 #endif
@@ -1052,34 +1152,41 @@ int
 ldap_msgdelete( LDAP *ld, int msgid )
 {
 	LDAPMessage	*lm, *prev;
+	int rc = 0;
 
 	assert( ld != NULL );
 
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "result", LDAP_LEVEL_ENTRY, "ldap_msgdelete\n" ));
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_msgdelete\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "ldap_msgdelete\n", 0, 0, 0 );
 #endif
 
 	prev = NULL;
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_res_mutex );
+#endif
 	for ( lm = ld->ld_responses; lm != NULL; lm = lm->lm_next ) {
 		if ( lm->lm_msgid == msgid )
 			break;
 		prev = lm;
 	}
 
-	if ( lm == NULL )
-		return( -1 );
+	if ( lm == NULL ) {
+		rc = -1;
+	} else {
+		if ( prev == NULL )
+			ld->ld_responses = lm->lm_next;
+		else
+			prev->lm_next = lm->lm_next;
+	}
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
+#endif
+	if ( lm && ldap_msgfree( lm ) == LDAP_RES_SEARCH_ENTRY )
+		rc = -1;
 
-	if ( prev == NULL )
-		ld->ld_responses = lm->lm_next;
-	else
-		prev->lm_next = lm->lm_next;
-
-	if ( ldap_msgfree( lm ) == LDAP_RES_SEARCH_ENTRY )
-		return( -1 );
-
-	return( 0 );
+	return( rc );
 }
 
 
