@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblunicode/ucdata/ucdata.c,v 1.15.2.2 2002/04/22 15:43:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblunicode/ucdata/ucdata.c,v 1.23.2.1 2003/09/18 15:43:32 kurt Exp $ */
 /*
- * Copyright 2000-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 2000-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -34,7 +34,9 @@
 #include <ac/string.h>
 #include <ac/unistd.h>
 
+#include <ac/bytes.h>
 
+#include "lber_pvt.h"
 #include "ucdata.h"
 
 /**************************************************************************
@@ -44,11 +46,11 @@
  **************************************************************************/
 
 typedef struct {
-    unsigned short bom;
-    unsigned short cnt;
+    ac_uint2 bom;
+    ac_uint2 cnt;
     union {
-        unsigned long bytes;
-        unsigned short len[2];
+        ac_uint4 bytes;
+        ac_uint2 len[2]; 
     } size;
 } _ucheader_t;
 
@@ -898,14 +900,14 @@ ucdecomp_hangul(unsigned long code, unsigned long *num, unsigned long decomp[])
 /* mode == 0 for canonical, mode == 1 for compatibility */
 static int
 uccanoncompatdecomp(const unsigned long *in, int inlen,
-		    unsigned long **out, int *outlen, short mode)
+		    unsigned long **out, int *outlen, short mode, void *ctx)
 {
     int l, size;
 	unsigned i, j, k;
     unsigned long num, class, *decomp, hangdecomp[3];
 
-    size = inlen;
-    *out = (unsigned long *) malloc(size * sizeof(**out));
+    size = inlen * 2;
+    *out = (unsigned long *) ber_memalloc_x(size * sizeof(**out), ctx);
     if (*out == NULL)
         return *outlen = -1;
 
@@ -914,7 +916,7 @@ uccanoncompatdecomp(const unsigned long *in, int inlen,
 	if (mode ? uckdecomp(in[j], &num, &decomp) : ucdecomp(in[j], &num, &decomp)) {
             if ( size - i < num) {
                 size = inlen + i - j + num - 1;
-                *out = (unsigned long *) realloc(*out, size * sizeof(**out));
+                *out = (unsigned long *) ber_memrealloc_x(*out, size * sizeof(**out), ctx );
                 if (*out == NULL)
                     return *outlen = -1;
             }
@@ -934,7 +936,7 @@ uccanoncompatdecomp(const unsigned long *in, int inlen,
         } else if (ucdecomp_hangul(in[j], &num, hangdecomp)) {
             if (size - i < num) {
                 size = inlen + i - j + num - 1;
-                *out = (unsigned long *) realloc(*out, size * sizeof(**out));
+                *out = (unsigned long *) ber_memrealloc_x(*out, size * sizeof(**out), ctx);
                 if (*out == NULL)
                     return *outlen = -1;
             }
@@ -945,7 +947,7 @@ uccanoncompatdecomp(const unsigned long *in, int inlen,
         } else {
             if (size - i < 1) {
                 size = inlen + i - j;
-                *out = (unsigned long *) realloc(*out, size * sizeof(**out));
+                *out = (unsigned long *) ber_memrealloc_x(*out, size * sizeof(**out), ctx);
                 if (*out == NULL)
                     return *outlen = -1;
             }
@@ -967,16 +969,16 @@ uccanoncompatdecomp(const unsigned long *in, int inlen,
 
 int
 uccanondecomp(const unsigned long *in, int inlen,
-              unsigned long **out, int *outlen)
+              unsigned long **out, int *outlen, void *ctx)
 {
-    return uccanoncompatdecomp(in, inlen, out, outlen, 0);
+    return uccanoncompatdecomp(in, inlen, out, outlen, 0, ctx);
 }
 
 int
 uccompatdecomp(const unsigned long *in, int inlen,
-	       unsigned long **out, int *outlen)
+	       unsigned long **out, int *outlen, void *ctx)
 {
-    return uccanoncompatdecomp(in, inlen, out, outlen, 1);
+    return uccanoncompatdecomp(in, inlen, out, outlen, 1, ctx);
 }
 
 /**************************************************************************
