@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblunicode/ucdata/ucdata.h,v 1.10.2.1 2002/04/22 15:43:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblunicode/ucdata/ucdata.h,v 1.13 2003/04/11 01:57:10 hyc Exp $ */
 /*
- * Copyright 2000-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 2000-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -273,7 +273,7 @@ ucdecomp_hangul LDAP_P((unsigned long code, unsigned long *num,
  */
 LDAP_LUNICODE_F (int)
 uccanondecomp LDAP_P((const unsigned long *in, int inlen,
-		     unsigned long **out, int *outlen));
+		     unsigned long **out, int *outlen, void *ctx));
   
 /*  
  * Equivalent to uccanondecomp() except that it includes compatibility
@@ -281,7 +281,7 @@ uccanondecomp LDAP_P((const unsigned long *in, int inlen,
  */
 LDAP_LUNICODE_F (int)
 uccompatdecomp LDAP_P((const unsigned long *in, int inlen,
-		     unsigned long **out, int *outlen));
+		     unsigned long **out, int *outlen, void *ctx));
   
 /**************************************************************************
  *
