@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/ava.c,v 1.26.2.1 2002/04/15 15:21:52 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/ava.c,v 1.34 2003/04/10 23:29:29 hyc Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /* ava.c - routines for dealing with attribute value assertions */
@@ -17,20 +17,22 @@
 
 void
 ava_free(
-    AttributeAssertion *ava,
-    int	freeit
+	Operation *op,
+	AttributeAssertion *ava,
+	int	freeit
 )
 {
-	free( ava->aa_value.bv_val );
+	op->o_tmpfree( ava->aa_value.bv_val, op->o_tmpmemctx );
 	if ( freeit ) {
-		ch_free( (char *) ava );
+		op->o_tmpfree( (char *) ava, op->o_tmpmemctx );
 	}
 }
 
 int
 get_ava(
-    BerElement	*ber,
-    AttributeAssertion	**ava,
+	Operation *op,
+	BerElement	*ber,
+	AttributeAssertion	**ava,
 	unsigned usage,
 	const char **text
 )
@@ -44,8 +46,7 @@ get_ava(
 
 	if( rtag == LBER_ERROR ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "filter", LDAP_LEVEL_ERR,
-			   "get_ava:  ber_scanf failure\n" ));
+		LDAP_LOG( FILTER, ERR, "get_ava:  ber_scanf failure\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "  get_ava ber_scanf\n", 0, 0, 0 );
 #endif
@@ -53,22 +54,23 @@ get_ava(
 		return SLAPD_DISCONNECT;
 	}
 
-	aa = ch_malloc( sizeof( AttributeAssertion ) );
+	aa = op->o_tmpalloc( sizeof( AttributeAssertion ), op->o_tmpmemctx );
 	aa->aa_desc = NULL;
 	aa->aa_value.bv_val = NULL;
 
 	rc = slap_bv2ad( &type, &aa->aa_desc, text );
 
 	if( rc != LDAP_SUCCESS ) {
-		ch_free( aa );
+		op->o_tmpfree( aa, op->o_tmpmemctx );
 		return rc;
 	}
 
-	rc = value_validate_normalize( aa->aa_desc, usage,
-		&value, &aa->aa_value, text );
+	rc = asserted_value_validate_normalize(
+		aa->aa_desc, ad_mr(aa->aa_desc, usage),
+		usage, &value, &aa->aa_value, text, op->o_tmpmemctx );
 
 	if( rc != LDAP_SUCCESS ) {
-		ch_free( aa );
+		op->o_tmpfree( aa, op->o_tmpmemctx );
 		return rc;
 	}
 
