@@ -1,7 +1,7 @@
 /* modify.c - ldbm backend modify routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/modify.c,v 1.88.2.1 2002/04/15 15:22:01 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/modify.c,v 1.100.2.1 2003/09/18 15:43:38 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -23,10 +23,7 @@
  * Juan C. Gomez (gomez@engr.sgi.com) 05/18/99
  */ 
 int ldbm_modify_internal(
-    Backend	*be,
-    Connection	*conn,
     Operation	*op,
-    const char	*dn,
     Modifications	*modlist,
     Entry	*e,
 	const char **text,
@@ -41,14 +38,13 @@ int ldbm_modify_internal(
 	Attribute 	*ap;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		"ldbm_modify_internal: %s\n", dn ));
+	LDAP_LOG( BACK_LDBM, ENTRY,  "ldbm_modify_internal: %s\n", e->e_name.bv_val, 0, 0 );
 #else
-	Debug(LDAP_DEBUG_TRACE, "ldbm_modify_internal: %s\n", dn, 0, 0);
+	Debug(LDAP_DEBUG_TRACE, "ldbm_modify_internal: %s\n", e->e_name.bv_val, 0, 0);
 #endif
 
 
-	if ( !acl_check_modlist( be, conn, op, e, modlist )) {
+	if ( !acl_check_modlist( op, e, modlist )) {
 		return LDAP_INSUFFICIENT_ACCESS;
 	}
 
@@ -61,18 +57,17 @@ int ldbm_modify_internal(
 		switch ( mod->sm_op ) {
 		case LDAP_MOD_ADD:
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-				"ldbm_modify_internal: add\n" ));
+			LDAP_LOG( BACK_LDBM, DETAIL1, "ldbm_modify_internal: add\n", 0, 0, 0);
 #else
 			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: add\n", 0, 0, 0);
 #endif
 
-			rc = modify_add_values( e, mod, text, textbuf, textlen );
+			rc = modify_add_values( e, mod, get_permissiveModify( op ),
+						text, textbuf, textlen );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-					"ldbm_modify_internal: failed %d (%s)\n",
-					rc, *text ));
+				LDAP_LOG( BACK_LDBM, INFO, 
+					"ldbm_modify_internal: failed %d (%s)\n", rc, *text, 0 );
 #else
 				Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: %d %s\n",
 					rc, *text, 0);
@@ -82,18 +77,18 @@ int ldbm_modify_internal(
 
 		case LDAP_MOD_DELETE:
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-				"ldbm_modify_internal: delete\n" ));
+			LDAP_LOG( BACK_LDBM, DETAIL1, "ldbm_modify_internal: delete\n", 0,0,0);
 #else
 			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: delete\n", 0, 0, 0);
 #endif
 
-			rc = modify_delete_values( e, mod, text, textbuf, textlen );
+			rc = modify_delete_values( e, mod, get_permissiveModify( op ),
+							text, textbuf, textlen );
 			assert( rc != LDAP_TYPE_OR_VALUE_EXISTS );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-					"ldbm_modify_internal: failed %d (%s)\n", rc, *text ));
+				LDAP_LOG( BACK_LDBM, INFO, 
+					"ldbm_modify_internal: failed %d (%s)\n", rc, *text, 0 );
 #else
 				Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: %d %s\n",
 					rc, *text, 0);
@@ -103,18 +98,39 @@ int ldbm_modify_internal(
 
 		case LDAP_MOD_REPLACE:
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-				"ldbm_modify_internal:  replace\n" ));
+			LDAP_LOG( BACK_LDBM, DETAIL1, "ldbm_modify_internal:  replace\n",0,0,0);
 #else
 			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: replace\n", 0, 0, 0);
 #endif
 
-			rc = modify_replace_values( e, mod, text, textbuf, textlen );
-			assert( rc != LDAP_TYPE_OR_VALUE_EXISTS );
+			rc = modify_replace_values( e, mod, get_permissiveModify( op ),
+							text, textbuf, textlen );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-					"ldbm_modify_internal: failed %d (%s)\n", rc, *text ));
+				LDAP_LOG( BACK_LDBM, INFO, 
+					"ldbm_modify_internal: failed %d (%s)\n", rc, *text, 0 );
+#else
+				Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: %d %s\n",
+					rc, *text, 0);
+#endif
+			}
+			break;
+
+		case LDAP_MOD_INCREMENT:
+#ifdef NEW_LOGGING
+			LDAP_LOG( BACK_LDBM, DETAIL1,
+				"ldbm_modify_internal:  increment\n",0,0,0);
+#else
+			Debug(LDAP_DEBUG_ARGS,
+				"ldbm_modify_internal:  increment\n",0,0,0);
+#endif
+
+			rc = modify_increment_values( e, mod, get_permissiveModify( op ),
+				text, textbuf, textlen );
+			if( rc != LDAP_SUCCESS ) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( BACK_LDBM, INFO, 
+					"ldbm_modify_internal: failed %d (%s)\n", rc, *text, 0 );
 #else
 				Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: %d %s\n",
 					rc, *text, 0);
@@ -124,8 +140,8 @@ int ldbm_modify_internal(
 
 		case SLAP_MOD_SOFTADD:
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-				"ldbm_modify_internal: softadd\n" ));
+			LDAP_LOG( BACK_LDBM, DETAIL1, 
+				"ldbm_modify_internal: softadd\n", 0, 0, 0 );
 #else
 			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: softadd\n", 0, 0, 0);
 #endif
@@ -135,15 +151,16 @@ int ldbm_modify_internal(
 			 */
 			mod->sm_op = LDAP_MOD_ADD;
 
-			rc = modify_add_values( e, mod, text, textbuf, textlen );
+			rc = modify_add_values( e, mod, get_permissiveModify( op ),
+						text, textbuf, textlen );
 			if ( rc == LDAP_TYPE_OR_VALUE_EXISTS ) {
 				rc = LDAP_SUCCESS;
 			}
 
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-					   "ldbm_modify_internal: failed %d (%s)\n", rc, *text ));
+				LDAP_LOG( BACK_LDBM, INFO, 
+					   "ldbm_modify_internal: failed %d (%s)\n", rc, *text, 0 );
 #else
 				Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: %d %s\n",
 					rc, *text, 0);
@@ -153,8 +170,8 @@ int ldbm_modify_internal(
 
 		default:
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-				"ldbm_modify_internal: invalid op %d\n", mod->sm_op ));
+			LDAP_LOG( BACK_LDBM, ERR, 
+				"ldbm_modify_internal: invalid op %d\n", mod->sm_op, 0, 0 );
 #else
 			Debug(LDAP_DEBUG_ANY, "ldbm_modify_internal: invalid op %d\n",
 				mod->sm_op, 0, 0);
@@ -163,8 +180,8 @@ int ldbm_modify_internal(
 			rc = LDAP_OTHER;
 			*text = "Invalid modify operation";
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-				"ldbm_modify_internal: %d (%s)\n", rc, *text ));
+			LDAP_LOG( BACK_LDBM, INFO, 
+				"ldbm_modify_internal: %d (%s)\n", rc, *text, 0 );
 #else
 			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: %d %s\n",
 				rc, *text, 0);
@@ -181,7 +198,7 @@ int ldbm_modify_internal(
 		}
 
 		/* check if modified attribute was indexed */
-		rc = index_is_indexed( be, mod->sm_desc );
+		rc = index_is_indexed( op->o_bd, mod->sm_desc );
 		if ( rc == LDAP_SUCCESS ) {
 			ap = attr_find( save_attrs, mod->sm_desc );
 			if ( ap ) ap->a_flags |= SLAP_ATTR_IXDEL;
@@ -192,12 +209,13 @@ int ldbm_modify_internal(
 	}
 
 	/* check that the entry still obeys the schema */
-	rc = entry_schema_check( be, e, save_attrs, text, textbuf, textlen );
+	rc = entry_schema_check( op->o_bd, e, save_attrs, text, textbuf, textlen );
+
 	if ( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			"ldbm_modify_internal: entry failed schema check: %s\n",
-			*text ));
+		LDAP_LOG( BACK_LDBM, ERR, 
+			"ldbm_modify_internal: entry failed schema check: %s\n", 
+			*text, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "entry failed schema check: %s\n",
 			*text, 0, 0 );
@@ -217,12 +235,14 @@ int ldbm_modify_internal(
 	/* start with deleting the old index entries */
 	for ( ap = save_attrs; ap != NULL; ap = ap->a_next ) {
 		if ( ap->a_flags & SLAP_ATTR_IXDEL ) {
-			rc = index_values( be, ap->a_desc, ap->a_vals, e->e_id,
-				  	   SLAP_INDEX_DELETE_OP );
+			rc = index_values( op, ap->a_desc,
+				ap->a_nvals,
+				e->e_id, SLAP_INDEX_DELETE_OP );
 			if ( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-				   	   "ldbm_modify_internal: Attribute index delete failure\n" ));
+				LDAP_LOG( BACK_LDBM, ERR,
+					"ldbm_modify_internal: Attribute index delete failure\n",
+					0, 0, 0	);
 #else
 				Debug( LDAP_DEBUG_ANY,
 				       "Attribute index delete failure",
@@ -237,12 +257,14 @@ int ldbm_modify_internal(
 	/* add the new index entries */
 	for ( ap = e->e_attrs; ap != NULL; ap = ap->a_next ) {
 		if ( ap->a_flags & SLAP_ATTR_IXADD ) {
-			rc = index_values( be, ap->a_desc, ap->a_vals, e->e_id,
-				  	   SLAP_INDEX_ADD_OP );
+			rc = index_values( op, ap->a_desc,
+				ap->a_nvals,
+				e->e_id, SLAP_INDEX_ADD_OP );
 			if ( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-				   	   "ldbm_modify_internal: Attribute index add failure\n" ));
+				LDAP_LOG( BACK_LDBM, ERR,
+					"ldbm_modify_internal: Attribute index add failure\n",
+					0, 0, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 				       "Attribute index add failure",
@@ -270,26 +292,18 @@ exit:
 
 int
 ldbm_back_modify(
-    Backend	*be,
-    Connection	*conn,
     Operation	*op,
-    struct berval	*dn,
-    struct berval	*ndn,
-    Modifications	*modlist
-)
+    SlapReply	*rs )
 {
-	int rc;
-	struct ldbminfo	*li = (struct ldbminfo *) be->be_private;
+	struct ldbminfo	*li = (struct ldbminfo *) op->o_bd->be_private;
 	Entry		*matched;
 	Entry		*e;
 	int		manageDSAit = get_manageDSAit( op );
-	const char *text = NULL;
 	char textbuf[SLAP_TEXT_BUFLEN];
 	size_t textlen = sizeof textbuf;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		"ldbm_back_modify: enter\n" ));
+	LDAP_LOG( BACK_LDBM, ENTRY, "ldbm_back_modify: enter\n", 0, 0, 0);
 #else
 	Debug(LDAP_DEBUG_ARGS, "ldbm_back_modify:\n", 0, 0, 0);
 #endif
@@ -298,83 +312,85 @@ ldbm_back_modify(
 	ldap_pvt_thread_rdwr_wlock(&li->li_giant_rwlock);
 
 	/* acquire and lock entry */
-	if ( (e = dn2entry_w( be, ndn, &matched )) == NULL ) {
-		char* matched_dn = NULL;
-		BerVarray refs;
+	e = dn2entry_w( op->o_bd, &op->o_req_ndn, &matched );
 
+	/* FIXME: dn2entry() should return non-glue entry */
+	if (( e == NULL ) || ( !manageDSAit && e && is_entry_glue( e ))) {
 		if ( matched != NULL ) {
-			matched_dn = ch_strdup( matched->e_dn );
-			refs = is_entry_referral( matched )
-				? get_entry_referrals( be, conn, op, matched )
+			rs->sr_matched = ch_strdup( matched->e_dn );
+			rs->sr_ref = is_entry_referral( matched )
+				? get_entry_referrals( op, matched )
 				: NULL;
 			cache_return_entry_r( &li->li_cache, matched );
 		} else {
-			refs = referral_rewrite( default_referral,
-				NULL, dn, LDAP_SCOPE_DEFAULT );
+			BerVarray deref = op->o_bd->syncinfo ?
+							  op->o_bd->syncinfo->provideruri_bv : default_referral;
+			rs->sr_ref = referral_rewrite( deref, NULL, &op->o_req_dn, LDAP_SCOPE_DEFAULT );
 		}
 
 		ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
-		send_ldap_result( conn, op, LDAP_REFERRAL,
-			matched_dn, NULL, refs, NULL );
+		rs->sr_err = LDAP_REFERRAL;
+		send_ldap_result( op, rs );
 
-		if ( refs ) ber_bvarray_free( refs );
-		free( matched_dn );
+		if ( rs->sr_ref ) ber_bvarray_free( rs->sr_ref );
+		free( (char *)rs->sr_matched );
 
-		return( -1 );
+		return rs->sr_err;
 	}
 
-    if ( !manageDSAit && is_entry_referral( e ) ) {
+	if ( !manageDSAit && is_entry_referral( e ) )
+	{
 		/* parent is a referral, don't allow add */
 		/* parent is an alias, don't allow add */
-		BerVarray refs = get_entry_referrals( be,
-			conn, op, e );
+		rs->sr_ref = get_entry_referrals( op, e );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-			   "ldbm_back_modify: entry (%s) is referral\n", ndn->bv_val ));
+		LDAP_LOG( BACK_LDBM, INFO, 
+			   "ldbm_back_modify: entry (%s) is referral\n", op->o_req_ndn.bv_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE, "entry is referral\n", 0,
 		    0, 0 );
 #endif
 
+		rs->sr_err = LDAP_REFERRAL;
+		rs->sr_matched = e->e_name.bv_val;
+		send_ldap_result( op, rs );
 
-		send_ldap_result( conn, op, LDAP_REFERRAL,
-		    e->e_dn, NULL, refs, NULL );
-
-		if ( refs ) ber_bvarray_free( refs );
+		if ( rs->sr_ref ) ber_bvarray_free( rs->sr_ref );
 
 		goto error_return;
 	}
 	
 	/* Modify the entry */
-	rc = ldbm_modify_internal( be, conn, op, ndn->bv_val, modlist, e,
-		&text, textbuf, textlen );
+	rs->sr_err = ldbm_modify_internal( op, op->oq_modify.rs_modlist, e,
+		&rs->sr_text, textbuf, textlen );
 
-	if( rc != LDAP_SUCCESS ) {
-		if( rc != SLAPD_ABANDON ) {
-			send_ldap_result( conn, op, rc,
-				NULL, text, NULL, NULL );
+	if( rs->sr_err != LDAP_SUCCESS ) {
+		if( rs->sr_err != SLAPD_ABANDON ) {
+			send_ldap_result( op, rs );
 		}
 
 		goto error_return;
 	}
 
 	/* change the entry itself */
-	if ( id2entry_add( be, e ) != 0 ) {
-		send_ldap_result( conn, op, LDAP_OTHER,
-			NULL, "id2entry failure", NULL, NULL );
+	if ( id2entry_add( op->o_bd, e ) != 0 ) {
+		send_ldap_error( op, rs, LDAP_OTHER,
+			"id2entry failure" );
+		rs->sr_err = LDAP_OTHER;
 		goto error_return;
 	}
 
-	send_ldap_result( conn, op, LDAP_SUCCESS,
-		NULL, NULL, NULL, NULL );
+	send_ldap_error( op, rs, LDAP_SUCCESS,
+		NULL );
 
 	cache_return_entry_w( &li->li_cache, e );
 	ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
-	return( 0 );
+
+	return LDAP_SUCCESS;
 
 error_return:;
 	cache_return_entry_w( &li->li_cache, e );
 	ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
-	return( -1 );
+	return rs->sr_err;
 }
