@@ -1,5 +1,5 @@
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  *
  * Copyright 1999, Howard Chu, All rights reserved. <hyc@highlandsun.com>
@@ -77,7 +77,7 @@
 
 /* String rewrite library */
 #include "rewrite.h"
-
+#include "cache.h"
 LDAP_BEGIN_DECL
 
 struct slap_conn;
@@ -87,15 +87,19 @@ struct metasingleconn {
 	int			candidate;
 #define	META_NOT_CANDIDATE	0
 #define	META_CANDIDATE		1
+#define	META_LAST_CONN		-1
 	
 	LDAP            	*ld;
 	struct berval          	bound_dn;
+	struct berval		cred;
 	int             	bound;
 #define META_UNBOUND		0
 #define META_BOUND		1
 #define META_ANONYMOUS		2
 };
 
+#define META_LAST(lsc)		((lsc)->candidate == META_LAST_CONN)
+
 struct metaconn {
 	struct slap_conn	*conn;
 	struct rewrite_info	*rwinfo;
@@ -108,7 +112,7 @@ struct metaconn {
 #define META_BOUND_NONE		-1
 #define META_BOUND_ALL		-2
 	/* supersedes the connection stuff */
-	struct metasingleconn **conns;
+	struct metasingleconn *conns;
 };
 
 struct metatarget {
@@ -121,10 +125,13 @@ struct metatarget {
 	struct berval           pseudorootdn;
 	struct berval           pseudorootpw;
 
+#if 0
 	struct rewrite_info	*rwinfo;
 
 	struct ldapmap		oc_map;
 	struct ldapmap		at_map;
+#endif
+	struct ldaprwmap	rwmap;
 };
 
 struct metadncache {
@@ -139,35 +146,29 @@ struct metadncache {
 struct metainfo {
 	int			ntargets;
 	int			defaulttarget;
+	int			network_timeout;
 #define META_DEFAULT_TARGET_NONE	-1
 	struct metatarget	**targets;
 
+	struct rewrite_info	*rwinfo;
+	cache_manager		*cm; 
+	Backend			*glue_be; 
+
 	struct metadncache	cache;
 	
 	ldap_pvt_thread_mutex_t	conn_mutex;
 	Avlnode			*conntree;
+
+	int			savecred;
 };
 
-extern int
-meta_back_do_single_bind(
-		struct metainfo         *li,
-		struct metaconn         *lc,
-		struct berval		*dn,
-		struct berval		*ndn,
-		struct berval		*cred,
-		int			method,
-		int                     candidate
-);
-
-
 #define META_OP_ALLOW_MULTIPLE		0x00
 #define META_OP_REQUIRE_SINGLE		0x01
 #define META_OP_REQUIRE_ALL		0x02
 extern struct metaconn *
 meta_back_getconn(
-		struct			metainfo *li,
-	       	struct			slap_conn *conn,
-		struct			slap_op *op,
+		Operation		*op,
+		SlapReply		*rs,
 		int			op_type,
 		struct berval		*dn,
 		int			*candidate
@@ -188,7 +189,8 @@ meta_back_is_valid(
 extern int
 meta_back_op_result(
 		struct metaconn		*lc,
-		Operation		*op
+		Operation		*op,
+		SlapReply		*rs
 );
 
 extern int
