@@ -1,5 +1,9 @@
 /* init.c - initialize passwd backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-passwd/init.c,v 1.6.4.5 2000/07/04 17:58:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-passwd/init.c,v 1.22.2.1 2003/09/18 15:43:39 kurt Exp $ */
+/*
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 
 #include "portable.h"
 
@@ -8,11 +12,13 @@
 #include <ac/socket.h>
 
 #include "slap.h"
-#include "external.h"
+#include "back-passwd.h"
+
+ldap_pvt_thread_mutex_t passwd_mutex;
 
 #ifdef SLAPD_PASSWD_DYNAMIC
 
-int back_passwd_LTX_init_module(int argc, char *argv[]) {
+int init_module(int argc, char *argv[]) {
     BackendInfo bi;
 
     memset( &bi, '\0', sizeof(bi) );
@@ -30,13 +36,15 @@ passwd_back_initialize(
     BackendInfo	*bi
 )
 {
+	ldap_pvt_thread_mutex_init( &passwd_mutex );
+
 	bi->bi_open = 0;
 	bi->bi_config = 0;
 	bi->bi_close = 0;
-	bi->bi_destroy = 0;
+	bi->bi_destroy = passwd_back_destroy;
 
 	bi->bi_db_init = 0;
-	bi->bi_db_config = 0;
+	bi->bi_db_config = passwd_back_db_config;
 	bi->bi_db_open = 0;
 	bi->bi_db_close = 0;
 	bi->bi_db_destroy = 0;
@@ -53,18 +61,19 @@ passwd_back_initialize(
 
 	bi->bi_extended = 0;
 
-	bi->bi_acl_group = 0;
-	bi->bi_acl_attribute = 0;
 	bi->bi_chk_referrals = 0;
 
-#ifdef HAVE_CYRUS_SASL
-	bi->bi_sasl_authorize = 0;
-	bi->bi_sasl_getsecret = 0;
-	bi->bi_sasl_putsecret = 0;
-#endif /* HAVE_CYRUS_SASL */
-
 	bi->bi_connection_init = 0;
 	bi->bi_connection_destroy = 0;
 
 	return 0;
 }
+
+int
+passwd_back_destroy(
+	BackendInfo *bi
+)
+{
+	ldap_pvt_thread_mutex_destroy( &passwd_mutex );
+	return 0;
+}
