@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/external.h,v 1.13 2002/01/04 20:17:54 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/external.h,v 1.14.2.1 2003/09/18 15:43:39 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 #ifndef _SHELL_EXTERNAL_H
@@ -34,8 +34,6 @@ extern BI_op_add	shell_back_add;
 
 extern BI_op_delete	shell_back_delete;
 
-extern BI_op_abandon	shell_back_abandon;
-
 LDAP_END_DECL
 
 #endif /* _SHELL_EXTERNAL_H */
