@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/connection.c,v 1.163.2.2 2002/04/18 15:39:22 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/connection.c,v 1.230.2.2 2003/09/18 15:43:34 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -19,6 +19,10 @@
 #include "lutil.h"
 #include "slap.h"
 
+#ifdef LDAP_SLAPI
+#include "slapi.h"
+#endif
+
 /* protected by connections_mutex */
 static ldap_pvt_thread_mutex_t connections_mutex;
 static Connection *connections = NULL;
@@ -55,15 +59,12 @@ static Connection* connection_get( ber_socket_t s );
 static int connection_input( Connection *c );
 static void connection_close( Connection *c );
 
-static int connection_op_activate( Connection *conn, Operation *op );
+static int connection_op_activate( Operation *op );
 static int connection_resched( Connection *conn );
 static void connection_abandon( Connection *conn );
 static void connection_destroy( Connection *c );
 
-struct co_arg {
-	Connection	*co_conn;
-	Operation	*co_op;
-};
+static ldap_pvt_thread_start_t connection_operation;
 
 /*
  * Initialize connection management infrastructure.
@@ -74,8 +75,8 @@ int connections_init(void)
 
 	if( connections != NULL) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-			   "connections_init:  already initialized.\n" ));
+		LDAP_LOG( CONNECTION, INFO,
+			   "connections_init:  already initialized.\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "connections_init: already initialized.\n",
 			0, 0, 0 );
@@ -86,13 +87,13 @@ int connections_init(void)
 	/* should check return of every call */
 	ldap_pvt_thread_mutex_init( &connections_mutex );
 
-	connections = (Connection *) calloc( dtblsize, sizeof(Connection) );
+	connections = (Connection *) ch_calloc( dtblsize, sizeof(Connection) );
 
 	if( connections == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connections_init: allocation (%d * %ld) of connection array failed\n",
-			   dtblsize, (long) sizeof(Connection) ));
+		LDAP_LOG( CONNECTION, ERR,
+			   "connections_init: allocation (%d * %ld) of connection "
+			   "array failed\n", dtblsize, (long) sizeof(Connection), 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"connections_init: allocation (%d*%ld) of connection array failed\n",
@@ -123,8 +124,8 @@ int connections_destroy(void)
 
 	if( connections == NULL) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-			   "connections_destroy: nothing to destroy.\n"));
+		LDAP_LOG( CONNECTION, INFO,
+			   "connections_destroy: nothing to destroy.\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "connections_destroy: nothing to destroy.\n",
 			0, 0, 0 );
@@ -138,6 +139,9 @@ int connections_destroy(void)
 			ldap_pvt_thread_mutex_destroy( &connections[i].c_mutex );
 			ldap_pvt_thread_mutex_destroy( &connections[i].c_write_mutex );
 			ldap_pvt_thread_cond_destroy( &connections[i].c_write_cv );
+#ifdef LDAP_SLAPI
+			slapi_x_free_object_extensions( SLAPI_X_EXT_CONNECTION, &connections[i] );
+#endif
 		}
 	}
 
@@ -208,8 +212,7 @@ static Connection* connection_get( ber_socket_t s )
 	Connection *c;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "connection", LDAP_LEVEL_ENTRY,
-		   "connection_get: socket %ld\n", (long)s ));
+	LDAP_LOG( CONNECTION, ENTRY, "connection_get: socket %ld\n", (long)s, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ARGS,
 		"connection_get(%ld)\n",
@@ -273,8 +276,8 @@ static Connection* connection_get( ber_socket_t s )
 			assert( sd == AC_SOCKET_INVALID );
 
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "connection", LDAP_LEVEL_ARGS,
-				   "connection_get:  connection %d not used\n", s ));
+			LDAP_LOG( CONNECTION, ARGS, 
+				"connection_get:  connection %d not used\n", s, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"connection_get(%d): connection not used\n",
@@ -286,8 +289,8 @@ static Connection* connection_get( ber_socket_t s )
 		}
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_RESULTS,
-			   "connection_get: get for %d got connid %lu\n", s, c->c_connid ));
+		LDAP_LOG( CONNECTION, RESULTS, 
+			"connection_get: get for %d got connid %lu\n", s, c->c_connid, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"connection_get(%d): got connid=%lu\n",
@@ -319,10 +322,9 @@ static void connection_return( Connection *c )
 
 long connection_init(
 	ber_socket_t s,
-	const char* url,
+	Listener *listener,
 	const char* dnsname,
 	const char* peername,
-	const char* sockname,
 	int tls_udp_option,
 	slap_ssf_t ssf,
 	const char *authid )
@@ -332,9 +334,9 @@ long connection_init(
 
 	assert( connections != NULL );
 
+	assert( listener != NULL );
 	assert( dnsname != NULL );
 	assert( peername != NULL );
-	assert( sockname != NULL );
 
 #ifndef HAVE_TLS
 	assert( tls_udp_option != 1 );
@@ -342,8 +344,8 @@ long connection_init(
 
 	if( s == AC_SOCKET_INVALID ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-			   "connection_init: init of socket %ld invalid.\n", (long)s ));
+		LDAP_LOG( CONNECTION, INFO, 
+			   "connection_init: init of socket %ld invalid.\n", (long)s, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		       "connection_init(%ld): invalid.\n",
@@ -394,49 +396,55 @@ long connection_init(
 
 	if( c == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-			   "connection_init: skt %d	 connection table full (%d/%d)\n",
-			   s, i, dtblsize ));
+		LDAP_LOG( CONNECTION, INFO, 
+			   "connection_init: skt %d connection table full "
+			   "(%d/%d)\n", s, i, dtblsize );
 #else
 		Debug( LDAP_DEBUG_ANY,
-				"connection_init(%d): connection table full (%d/%d)\n",
-				s, i, dtblsize);
+				"connection_init(%d): connection table full "
+				"(%d/%d)\n", s, i, dtblsize);
 #endif
 	    ldap_pvt_thread_mutex_unlock( &connections_mutex );
 	    return -1;
 	}
-    }
+    	}
 #endif
 
-    assert( c != NULL );
+	assert( c != NULL );
 
 	if( c->c_struct_state == SLAP_C_UNINITIALIZED ) {
+		c->c_send_ldap_result = slap_send_ldap_result;
+		c->c_send_search_entry = slap_send_search_entry;
+		c->c_send_search_reference = slap_send_search_reference;
+		c->c_send_ldap_extended = slap_send_ldap_extended;
+#ifdef LDAP_RES_INTERMEDIATE
+		c->c_send_ldap_intermediate = slap_send_ldap_intermediate;
+#endif
+
 		c->c_authmech.bv_val = NULL;
 		c->c_authmech.bv_len = 0;
 		c->c_dn.bv_val = NULL;
 		c->c_dn.bv_len = 0;
 		c->c_ndn.bv_val = NULL;
 		c->c_ndn.bv_len = 0;
-		c->c_cdn.bv_val = NULL;
-		c->c_cdn.bv_len = 0;
 		c->c_groups = NULL;
 
-		c->c_listener_url.bv_val = NULL;
-		c->c_listener_url.bv_len = 0;
+		c->c_listener = NULL;
 		c->c_peer_domain.bv_val = NULL;
 		c->c_peer_domain.bv_len = 0;
 		c->c_peer_name.bv_val = NULL;
 		c->c_peer_name.bv_len = 0;
-		c->c_sock_name.bv_val = NULL;
-		c->c_sock_name.bv_len = 0;
 
 		LDAP_STAILQ_INIT(&c->c_ops);
 		LDAP_STAILQ_INIT(&c->c_pending_ops);
 
 		c->c_sasl_bind_mech.bv_val = NULL;
 		c->c_sasl_bind_mech.bv_len = 0;
-		c->c_sasl_context = NULL;
+		c->c_sasl_done = 0;
+		c->c_sasl_authctx = NULL;
+		c->c_sasl_sockctx = NULL;
 		c->c_sasl_extra = NULL;
+		c->c_sasl_bindop = NULL;
 
 		c->c_sb = ber_sockbuf_alloc( );
 
@@ -452,6 +460,10 @@ long connection_init(
 		ldap_pvt_thread_mutex_init( &c->c_write_mutex );
 		ldap_pvt_thread_cond_init( &c->c_write_cv );
 
+#ifdef LDAP_SLAPI
+		slapi_x_create_object_extensions( SLAPI_X_EXT_CONNECTION, c );
+#endif
+
 		c->c_struct_state = SLAP_C_UNUSED;
 	}
 
@@ -461,23 +473,23 @@ long connection_init(
     assert( c->c_authmech.bv_val == NULL );
     assert( c->c_dn.bv_val == NULL );
     assert( c->c_ndn.bv_val == NULL );
-    assert( c->c_cdn.bv_val == NULL );
     assert( c->c_groups == NULL );
-    assert( c->c_listener_url.bv_val == NULL );
+    assert( c->c_listener == NULL );
     assert( c->c_peer_domain.bv_val == NULL );
     assert( c->c_peer_name.bv_val == NULL );
-    assert( c->c_sock_name.bv_val == NULL );
     assert( LDAP_STAILQ_EMPTY(&c->c_ops) );
     assert( LDAP_STAILQ_EMPTY(&c->c_pending_ops) );
 	assert( c->c_sasl_bind_mech.bv_val == NULL );
-	assert( c->c_sasl_context == NULL );
+	assert( c->c_sasl_done == 0 );
+	assert( c->c_sasl_authctx == NULL );
+	assert( c->c_sasl_sockctx == NULL );
 	assert( c->c_sasl_extra == NULL );
+	assert( c->c_sasl_bindop == NULL );
 	assert( c->c_currentber == NULL );
 
-	ber_str2bv( url, 0, 1, &c->c_listener_url );
+	c->c_listener = listener;
 	ber_str2bv( dnsname, 0, 1, &c->c_peer_domain );
 	ber_str2bv( peername, 0, 1, &c->c_peer_name );
-	ber_str2bv( sockname, 0, 1, &c->c_sock_name );
 
     c->c_n_ops_received = 0;
     c->c_n_ops_executing = 0;
@@ -501,26 +513,25 @@ long connection_init(
 
 #ifdef LDAP_CONNECTIONLESS
 	c->c_is_udp = 0;
-	if (tls_udp_option == 2)
-	{
+	if( tls_udp_option == 2 ) {
 		c->c_is_udp = 1;
 #ifdef LDAP_DEBUG
-	ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_debug,
-		LBER_SBIOD_LEVEL_PROVIDER, (void*)"udp_" );
+		ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_debug,
+			LBER_SBIOD_LEVEL_PROVIDER, (void*)"udp_" );
 #endif
-	ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_udp,
-		LBER_SBIOD_LEVEL_PROVIDER, (void *)&s );
-	ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_readahead,
-		LBER_SBIOD_LEVEL_PROVIDER, NULL );
+		ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_udp,
+			LBER_SBIOD_LEVEL_PROVIDER, (void *)&s );
+		ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_readahead,
+			LBER_SBIOD_LEVEL_PROVIDER, NULL );
 	} else
 #endif
 	{
 #ifdef LDAP_DEBUG
-	ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_debug,
-		LBER_SBIOD_LEVEL_PROVIDER, (void*)"tcp_" );
+		ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_debug,
+			LBER_SBIOD_LEVEL_PROVIDER, (void*)"tcp_" );
 #endif
-	ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_tcp,
-		LBER_SBIOD_LEVEL_PROVIDER, (void *)&s );
+		ber_sockbuf_add_io( c->c_sb, &ber_sockbuf_io_tcp,
+			LBER_SBIOD_LEVEL_PROVIDER, (void *)&s );
 	}
 
 #ifdef LDAP_DEBUG
@@ -532,9 +543,9 @@ long connection_init(
 		c /* non-NULL */ ) < 0 )
 	{
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
+		LDAP_LOG( CONNECTION, INFO, 
 			   "connection_init: conn %lu  set nonblocking failed\n",
-			   c->c_connid ));
+			   c->c_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"connection_init(%d, %s): set nonblocking failed\n",
@@ -560,7 +571,7 @@ long connection_init(
     }
 #endif
 
-	slap_sasl_open( c );
+	slap_sasl_open( c, 0 );
 	slap_sasl_external( c, ssf, authid );
 
     ldap_pvt_thread_mutex_unlock( &c->c_mutex );
@@ -573,8 +584,8 @@ long connection_init(
 
 void connection2anonymous( Connection *c )
 {
-    assert( connections != NULL );
-    assert( c != NULL );
+	assert( connections != NULL );
+	assert( c != NULL );
 
 	{
 		ber_len_t max = sockbuf_max_incoming;
@@ -587,35 +598,27 @@ void connection2anonymous( Connection *c )
 	}
 	c->c_authmech.bv_len = 0;
 
-    if(c->c_dn.bv_val != NULL) {
-	free(c->c_dn.bv_val);
-	c->c_dn.bv_val = NULL;
-    }
-    c->c_dn.bv_len = 0;
-    if(c->c_ndn.bv_val != NULL) {
-	free(c->c_ndn.bv_val);
-	c->c_ndn.bv_val = NULL;
-    }
-    c->c_ndn.bv_len = 0;
-
-	if(c->c_cdn.bv_val != NULL) {
-		free(c->c_cdn.bv_val);
-		c->c_cdn.bv_val = NULL;
+	if(c->c_dn.bv_val != NULL) {
+		free(c->c_dn.bv_val);
+		c->c_dn.bv_val = NULL;
 	}
-	c->c_cdn.bv_len = 0;
+	c->c_dn.bv_len = 0;
+	if(c->c_ndn.bv_val != NULL) {
+		free(c->c_ndn.bv_val);
+		c->c_ndn.bv_val = NULL;
+	}
+	c->c_ndn.bv_len = 0;
 
 	c->c_authz_backend = NULL;
-    
-    {
-	GroupAssertion *g, *n;
-	for (g = c->c_groups; g; g=n)
+	
 	{
-	    n = g->ga_next;
-	    free(g);
+		GroupAssertion *g, *n;
+		for (g = c->c_groups; g; g=n) {
+			n = g->ga_next;
+			free(g);
+		}
+		c->c_groups = NULL;
 	}
-	c->c_groups = NULL;
-    }
-
 }
 
 static void
@@ -642,12 +645,7 @@ connection_destroy( Connection *c )
     c->c_activitytime = c->c_starttime = 0;
 
 	connection2anonymous( c );
-
-	if(c->c_listener_url.bv_val != NULL) {
-		free(c->c_listener_url.bv_val);
-		c->c_listener_url.bv_val = NULL;
-	}
-	c->c_listener_url.bv_len = 0;
+	c->c_listener = NULL;
 
 	if(c->c_peer_domain.bv_val != NULL) {
 		free(c->c_peer_domain.bv_val);
@@ -655,31 +653,10 @@ connection_destroy( Connection *c )
 	}
 	c->c_peer_domain.bv_len = 0;
 	if(c->c_peer_name.bv_val != NULL) {
-#ifdef LDAP_PF_LOCAL
-		/*
-		 * If peer was a domain socket, unlink. Mind you,
-		 * they may be un-named. Should we leave this to
-		 * the client?
-		 */
-		if (strncmp(c->c_peer_name.bv_val, "PATH=", 
-					sizeof("PATH=") - 1) == 0) {
-			char *path = c->c_peer_name.bv_val 
-				+ sizeof("PATH=") - 1;
-			if (path[0] != '\0') {
-				(void)unlink(path);
-			}
-		}
-#endif /* LDAP_PF_LOCAL */
-
 		free(c->c_peer_name.bv_val);
 		c->c_peer_name.bv_val = NULL;
 	}
 	c->c_peer_name.bv_len = 0;
-	if(c->c_sock_name.bv_val != NULL) {
-		free(c->c_sock_name.bv_val);
-		c->c_sock_name.bv_val = NULL;
-	}
-	c->c_sock_name.bv_len = 0;
 
 	c->c_sasl_bind_in_progress = 0;
 	if(c->c_sasl_bind_mech.bv_val != NULL) {
@@ -700,8 +677,8 @@ connection_destroy( Connection *c )
 		slapd_remove( sd, 0 );
 
 		Statslog( LDAP_DEBUG_STATS,
-		    "conn=%lu fd=%d closed\n",
-			connid, sd, 0, 0, 0 );
+		    "conn=%lu fd=%ld closed\n",
+			connid, (long) sd, 0, 0, 0 );
 	}
 
 	ber_sockbuf_free( c->c_sb );
@@ -715,6 +692,11 @@ connection_destroy( Connection *c )
 
     c->c_conn_state = SLAP_C_INVALID;
     c->c_struct_state = SLAP_C_UNUSED;
+
+#ifdef LDAP_SLAPI
+	/* call destructors, then constructors; avoids unnecessary allocation */
+	slapi_x_clear_object_extensions( SLAPI_X_EXT_CONNECTION, c );
+#endif
 }
 
 int connection_state_closing( Connection *c )
@@ -764,9 +746,9 @@ void connection_closing( Connection *c )
 
 		ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_GET_FD, &sd );
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_DETAIL1,
+		LDAP_LOG( CONNECTION, DETAIL1, 
 			   "connection_closing: conn %lu readying socket %d for close.\n",
-			   c->c_connid, sd ));
+			   c->c_connid, sd, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"connection_closing: readying conn=%lu sd=%d for close\n",
@@ -801,9 +783,9 @@ static void connection_close( Connection *c )
 	ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_GET_FD, &sd );
 	if( !LDAP_STAILQ_EMPTY(&c->c_ops) ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_DETAIL1,
+		LDAP_LOG( CONNECTION, DETAIL1, 
 			   "connection_close: conn %lu  deferring sd %d\n",
-			   c->c_connid, sd ));
+			   c->c_connid, sd, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"connection_close: deferring conn=%lu sd=%d\n",
@@ -813,9 +795,8 @@ static void connection_close( Connection *c )
 	}
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "connection", LDAP_LEVEL_RESULTS,
-		   "connection_close: conn %lu  sd %d\n",
-		   c->c_connid, sd ));
+	LDAP_LOG( CONNECTION, RESULTS, 
+		   "connection_close: conn %lu  sd %d\n", c->c_connid, sd, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "connection_close: conn=%lu sd=%d\n",
 		c->c_connid, sd, 0 );
@@ -917,99 +898,125 @@ void connection_done( Connection *c )
 #endif /* !SLAPD_MONITOR */
 
 static void *
-connection_operation( void *arg_v )
+connection_operation( void *ctx, void *arg_v )
 {
-	int rc;
-	struct co_arg	*arg = arg_v;
-	ber_tag_t tag = arg->co_op->o_tag;
+	int rc = SLAPD_DISCONNECT;
+	Operation *op = arg_v;
+	SlapReply rs = {REP_RESULT};
+	ber_tag_t tag = op->o_tag;
 #ifdef SLAPD_MONITOR
 	ber_tag_t oldtag = tag;
 #endif /* SLAPD_MONITOR */
-	Connection *conn = arg->co_conn;
+	Connection *conn = op->o_conn;
+	void *memctx = NULL;
+	ber_len_t memsiz;
 
 	ldap_pvt_thread_mutex_lock( &num_ops_mutex );
 	num_ops_initiated++;
 	ldap_pvt_thread_mutex_unlock( &num_ops_mutex );
 
+	op->o_threadctx = ctx;
+
 	if( conn->c_sasl_bind_in_progress && tag != LDAP_REQ_BIND ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connection_operation: conn %lu  SASL bind in progress (tag=%ld).\n",
-			   conn->c_connid, (long)tag ));
+		LDAP_LOG( CONNECTION, ERR, 
+			"connection_operation: conn %lu SASL bind in progress (tag=%ld).\n",
+			conn->c_connid, (long)tag, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "connection_operation: "
 			"error: SASL bind in progress (tag=%ld).\n",
 			(long) tag, 0, 0 );
 #endif
-		send_ldap_result( conn, arg->co_op,
-			rc = LDAP_OPERATIONS_ERROR,
-			NULL, "SASL bind in progress", NULL, NULL );
+		send_ldap_error( op, &rs, LDAP_OPERATIONS_ERROR,
+			"SASL bind in progress" );
 		goto operations_error;
 	}
 
+	/* We can use Thread-Local storage for most mallocs. We can
+	 * also use TL for ber parsing, but not on Add or Modify.
+	 */
+#define	SLAB_SIZE	1048576
+#if 0
+	memsiz = ber_len( op->o_ber ) * 64;
+	if ( SLAB_SIZE > memsiz ) memsiz = SLAB_SIZE;
+#endif
+	memsiz = SLAB_SIZE;
+
+	memctx = sl_mem_create( memsiz, ctx );
+	op->o_tmpmemctx = memctx;
+	op->o_tmpmfuncs = &sl_mfuncs;
+	if ( tag != LDAP_REQ_ADD && tag != LDAP_REQ_MODIFY ) {
+		/* Note - the ber and its buffer are already allocated from
+		 * regular memory; this only affects subsequent mallocs that
+		 * ber_scanf may invoke.
+		 */
+		ber_set_option( op->o_ber, LBER_OPT_BER_MEMCTX, &memctx );
+	}
+
 	switch ( tag ) {
 	case LDAP_REQ_BIND:
 		INCR_OP(num_ops_initiated_, SLAP_OP_BIND);
-		rc = do_bind( conn, arg->co_op );
+		rc = do_bind( op, &rs );
 		break;
 
 	case LDAP_REQ_UNBIND:
 		INCR_OP(num_ops_initiated_, SLAP_OP_UNBIND);
-		rc = do_unbind( conn, arg->co_op );
+		rc = do_unbind( op, &rs );
 		break;
 
 	case LDAP_REQ_ADD:
 		INCR_OP(num_ops_initiated_, SLAP_OP_ADD);
-		rc = do_add( conn, arg->co_op );
+		rc = do_add( op, &rs );
 		break;
 
 	case LDAP_REQ_DELETE:
 		INCR_OP(num_ops_initiated_, SLAP_OP_DELETE);
-		rc = do_delete( conn, arg->co_op );
+		rc = do_delete( op, &rs );
 		break;
 
 	case LDAP_REQ_MODRDN:
 		INCR_OP(num_ops_initiated_, SLAP_OP_MODRDN);
-		rc = do_modrdn( conn, arg->co_op );
+		rc = do_modrdn( op, &rs );
 		break;
 
 	case LDAP_REQ_MODIFY:
 		INCR_OP(num_ops_initiated_, SLAP_OP_MODIFY);
-		rc = do_modify( conn, arg->co_op );
+		rc = do_modify( op, &rs );
 		break;
 
 	case LDAP_REQ_COMPARE:
 		INCR_OP(num_ops_initiated_, SLAP_OP_COMPARE);
-		rc = do_compare( conn, arg->co_op );
+		rc = do_compare( op, &rs );
 		break;
 
 	case LDAP_REQ_SEARCH:
 		INCR_OP(num_ops_initiated_, SLAP_OP_SEARCH);
-		rc = do_search( conn, arg->co_op );
+		rc = do_search( op, &rs );
 		break;
 
 	case LDAP_REQ_ABANDON:
 		INCR_OP(num_ops_initiated_, SLAP_OP_ABANDON);
-		rc = do_abandon( conn, arg->co_op );
+		rc = do_abandon( op, &rs );
 		break;
 
 	case LDAP_REQ_EXTENDED:
 		INCR_OP(num_ops_initiated_, SLAP_OP_EXTENDED);
-		rc = do_extended( conn, arg->co_op );
+		rc = do_extended( op, &rs );
 		break;
 
 	default:
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
+		LDAP_LOG( CONNECTION, INFO, 
 			   "connection_operation: conn %lu  unknown LDAP request 0x%lx\n",
-			   conn->c_connid, tag ));
+			   conn->c_connid, tag, 0  );
 #else
 		Debug( LDAP_DEBUG_ANY, "unknown LDAP request 0x%lx\n",
 		    tag, 0, 0 );
 #endif
-		arg->co_op->o_tag = LBER_ERROR;
-		send_ldap_disconnect( conn, arg->co_op,
-			LDAP_PROTOCOL_ERROR, "unknown LDAP request" );
+		op->o_tag = LBER_ERROR;
+		rs.sr_err = LDAP_PROTOCOL_ERROR;
+		rs.sr_text = "unknown LDAP request";
+		send_ldap_disconnect( op, &rs );
 		rc = -1;
 		break;
 	}
@@ -1058,18 +1065,40 @@ operations_error:
 #endif /* SLAPD_MONITOR */
 	ldap_pvt_thread_mutex_unlock( &num_ops_mutex );
 
+#ifdef LDAP_EXOP_X_CANCEL
+	if ( op->o_cancel == SLAP_CANCEL_REQ ) {
+		op->o_cancel = LDAP_TOO_LATE;
+	}
+
+	while ( op->o_cancel != SLAP_CANCEL_NONE &&
+		op->o_cancel != SLAP_CANCEL_DONE )
+	{
+		ldap_pvt_thread_yield();
+	}
+#endif
+
 	ldap_pvt_thread_mutex_lock( &conn->c_mutex );
 
 	conn->c_n_ops_executing--;
 	conn->c_n_ops_completed++;
 
-	LDAP_STAILQ_REMOVE( &conn->c_ops, arg->co_op, slap_op, o_next);
-	LDAP_STAILQ_NEXT(arg->co_op, o_next) = NULL;
-	slap_op_free( arg->co_op );
-	arg->co_op = NULL;
-	arg->co_conn = NULL;
-	free( (char *) arg );
-	arg = NULL;
+	LDAP_STAILQ_REMOVE( &conn->c_ops, op, slap_op, o_next);
+	LDAP_STAILQ_NEXT(op, o_next) = NULL;
+
+	if ( op->o_cancel == SLAP_CANCEL_ACK )
+		goto co_op_free;
+	if ( ( op->o_sync_mode & SLAP_SYNC_PERSIST ) ) {
+		sl_mem_detach( ctx, memctx );
+		goto no_co_op_free;
+	}
+
+co_op_free:
+
+	memctx = NULL;
+	ber_set_option( op->o_ber, LBER_OPT_BER_MEMCTX, &memctx );
+	slap_op_free( op );
+
+no_co_op_free:
 
 	switch( tag ) {
 	case LBER_ERROR:
@@ -1098,6 +1127,7 @@ int connection_read(ber_socket_t s)
 {
 	int rc = 0;
 	Connection *c;
+
 	assert( connections != NULL );
 
 	ldap_pvt_thread_mutex_lock( &connections_mutex );
@@ -1107,9 +1137,8 @@ int connection_read(ber_socket_t s)
 
 	if( c == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-			   "connection_read: sock %ld no connection\n",
-			   (long)s ));
+		LDAP_LOG( CONNECTION, INFO, 
+			"connection_read: sock %ld no connection\n", (long)s, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"connection_read(%ld): no connection!\n",
@@ -1125,9 +1154,9 @@ int connection_read(ber_socket_t s)
 
 	if( c->c_conn_state == SLAP_C_CLOSING ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-			   "connection_read: conn %lu connection closing, ignoring input\n",
-			   c->c_connid));
+		LDAP_LOG( CONNECTION, INFO, 
+			"connection_read: conn %lu connection closing, ignoring input\n",
+			c->c_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"connection_read(%d): closing, ignoring input for id=%lu\n",
@@ -1139,8 +1168,9 @@ int connection_read(ber_socket_t s)
 	}
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "connection", LDAP_LEVEL_DETAIL1,
-		   "connection_read: conn %lu  checking for input.\n", c->c_connid ));
+	LDAP_LOG( CONNECTION, DETAIL1, 
+		   "connection_read: conn %lu  checking for input.\n", 
+		   c->c_connid, 0, 0  );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"connection_read(%d): checking for input on id=%lu\n",
@@ -1157,9 +1187,9 @@ int connection_read(ber_socket_t s)
 #endif
 
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
+			LDAP_LOG( CONNECTION, ERR, 
 				   "connection_read: conn %lu  TLS accept error, error %d\n",
-				   c->c_connid, rc ));
+				   c->c_connid, rc, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"connection_read(%d): TLS accept error "
@@ -1175,21 +1205,20 @@ int connection_read(ber_socket_t s)
 			 * to propagate to client. */
 			FD_ZERO(&rfd);
 			FD_SET(s, &rfd);
-			for (rc=1; rc>0;)
-			{
+			for (rc=1; rc>0;) {
 			    tv.tv_sec = 1;
 			    tv.tv_usec = 0;
 			    rc = select(s+1, &rfd, NULL, NULL, &tv);
-			    if (rc == 1)
-				ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_DRAIN,
-				    NULL);
+			    if (rc == 1) {
+					ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_DRAIN, NULL);
+				}
 			}
 #endif
 			connection_close( c );
 
 		} else if ( rc == 0 ) {
 			void *ssl;
-			char *authid;
+			struct berval authid = { 0, NULL };
 
 			c->c_needs_tls_accept = 0;
 
@@ -1201,27 +1230,52 @@ int connection_read(ber_socket_t s)
 				c->c_ssf = c->c_tls_ssf;
 			}
 
-			authid = dnX509peerNormalize( ssl );
-			slap_sasl_external( c, c->c_tls_ssf, authid );
-			if ( authid )	free( authid );
+			rc = dnX509peerNormalize( ssl, &authid );
+			if ( rc != LDAP_SUCCESS ) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( CONNECTION, INFO, 
+					"connection_read: conn %lu unable to get TLS client DN, "
+					"error %d\n", c->c_connid, rc, 0 );
+#else
+				Debug( LDAP_DEBUG_TRACE,
+				"connection_read(%d): unable to get TLS client DN "
+				"error=%d id=%lu\n",
+				s, rc, c->c_connid );
+#endif
+			}
+			slap_sasl_external( c, c->c_tls_ssf, authid.bv_val );
+			if ( authid.bv_val )	free( authid.bv_val );
+		}
+
+		/* if success and data is ready, fall thru to data input loop */
+		if( rc != 0 ||
+			!ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_DATA_READY, NULL ) )
+		{
+			connection_return( c );
+			ldap_pvt_thread_mutex_unlock( &connections_mutex );
+			return 0;
 		}
-		connection_return( c );
-		ldap_pvt_thread_mutex_unlock( &connections_mutex );
-		return 0;
 	}
 #endif
 
 #ifdef HAVE_CYRUS_SASL
 	if ( c->c_sasl_layers ) {
+		/* If previous layer is not removed yet, give up for now */
+		if ( !c->c_sasl_sockctx ) {
+			connection_return( c );
+			ldap_pvt_thread_mutex_unlock( &connections_mutex );
+			return 0;
+		}
+
 		c->c_sasl_layers = 0;
 
-		rc = ldap_pvt_sasl_install( c->c_sb,  c->c_sasl_context );
+		rc = ldap_pvt_sasl_install( c->c_sb,  c->c_sasl_sockctx );
 
 		if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-				   "connection_read: conn %lu SASL install error %d, closing\n",
-				   c->c_connid, rc ));
+			LDAP_LOG( CONNECTION, ERR, 
+				"connection_read: conn %lu SASL install error %d, closing\n",
+				c->c_connid, rc, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"connection_read(%d): SASL install error "
@@ -1239,22 +1293,26 @@ int connection_read(ber_socket_t s)
 #endif
 
 #define CONNECTION_INPUT_LOOP 1
+/* #define	DATA_READY_LOOP 1 */
 
-#ifdef DATA_READY_LOOP
-	while( !rc && ber_sockbuf_ctrl( c->c_sb, LBER_SB_DATA_READY, NULL ) )
-#elif CONNECTION_INPUT_LOOP
-	while(!rc)
-#endif
+	do
 	{
 		/* How do we do this without getting into a busy loop ? */
 		rc = connection_input( c );
 	}
+#ifdef DATA_READY_LOOP
+	while( !rc && ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_DATA_READY, NULL ) );
+#elif CONNECTION_INPUT_LOOP
+	while(!rc);
+#else
+	while(0);
+#endif
 
 	if( rc < 0 ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connection_read: conn %lu  input error %d, closing.\n",
-			   c->c_connid, rc ));
+		LDAP_LOG( CONNECTION, ERR, 
+			"connection_read: conn %lu  input error %d, closing.\n",
+			c->c_connid, rc, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"connection_read(%d): input error=%d id=%lu, closing.\n",
@@ -1291,17 +1349,19 @@ connection_input(
 	ber_len_t	len;
 	ber_int_t	msgid;
 	BerElement	*ber;
+	int 		rc;
 #ifdef LDAP_CONNECTIONLESS
 	Sockaddr	peeraddr;
 	char 		*cdn = NULL;
 #endif
 
-	if ( conn->c_currentber == NULL && (conn->c_currentber = ber_alloc())
-	    == NULL ) {
+	if ( conn->c_currentber == NULL &&
+		( conn->c_currentber = ber_alloc()) == NULL )
+	{
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connection_input: conn %lu  ber_alloc failed.\n", 
-			   conn->c_connid ));
+		LDAP_LOG( CONNECTION, ERR, 
+			"connection_input: conn %lu  ber_alloc failed.\n", 
+			conn->c_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "ber_alloc failed\n", 0, 0, 0 );
 #endif
@@ -1311,8 +1371,7 @@ connection_input(
 	errno = 0;
 
 #ifdef LDAP_CONNECTIONLESS
-	if (conn->c_is_udp)
-	{
+	if ( conn->c_is_udp ) {
 		char	peername[sizeof("IP=255.255.255.255:65336")];
 		len = ber_int_sb_read(conn->c_sb, &peeraddr,
 			sizeof(struct sockaddr));
@@ -1334,9 +1393,9 @@ connection_input(
 		ber_sockbuf_ctrl( conn->c_sb, LBER_SB_OPT_GET_FD, &sd );
 
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connection_input: conn %lu  ber_get_next failed, errno %d (%s).\n",
-			   conn->c_connid, err, sock_errstr(err) ));
+		LDAP_LOG( CONNECTION, ERR, 
+			"connection_input: conn %lu  ber_get_next failed, errno %d (%s).\n",
+			conn->c_connid, err, sock_errstr(err) );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"ber_get_next on fd %d failed errno=%d (%s)\n",
@@ -1358,9 +1417,9 @@ connection_input(
 	if ( (tag = ber_get_int( ber, &msgid )) != LDAP_TAG_MSGID ) {
 		/* log, close and send error */
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connection_input: conn %lu  ber_get_int returns 0x%lx.\n",
-			   conn->c_connid, tag ));
+		LDAP_LOG( CONNECTION, ERR, 
+			"connection_input: conn %lu  ber_get_int returns 0x%lx.\n",
+			conn->c_connid, tag, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "ber_get_int returns 0x%lx\n", tag, 0,
 		    0 );
@@ -1372,9 +1431,9 @@ connection_input(
 	if ( (tag = ber_peek_tag( ber, &len )) == LBER_ERROR ) {
 		/* log, close and send error */
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
+		LDAP_LOG( CONNECTION, ERR, 
 			   "connection_input: conn %lu  ber_peek_tag returns 0x%lx.\n",
-			   conn->c_connid, tag ));
+			   conn->c_connid, tag, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "ber_peek_tag returns 0x%lx\n", tag, 0,
 		    0 );
@@ -1385,16 +1444,16 @@ connection_input(
 	}
 
 #ifdef LDAP_CONNECTIONLESS
-	if (conn->c_is_udp) {
-		if (tag == LBER_OCTETSTRING) {
+	if( conn->c_is_udp ) {
+		if( tag == LBER_OCTETSTRING ) {
 			ber_get_stringa( ber, &cdn );
 			tag = ber_peek_tag(ber, &len);
 		}
-		if (tag != LDAP_REQ_ABANDON && tag != LDAP_REQ_SEARCH) {
+		if( tag != LDAP_REQ_ABANDON && tag != LDAP_REQ_SEARCH ) {
 #ifdef NEW_LOGGING
-		    LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
+		    LDAP_LOG( CONNECTION, ERR, 
 			       "connection_input: conn %lu  invalid req for UDP 0x%lx.\n",
-			       conn->c_connid, tag ));
+			       conn->c_connid, tag, 0 );
 #else
 		    Debug( LDAP_DEBUG_ANY, "invalid req for UDP 0x%lx\n", tag, 0,
 			0 );
@@ -1411,31 +1470,92 @@ connection_input(
 
 	op = slap_op_alloc( ber, msgid, tag, conn->c_n_ops_received++ );
 
+	op->o_conn = conn;
+	op->o_assertion = NULL;
+	op->o_preread_attrs = NULL;
+	op->o_postread_attrs = NULL;
+	op->o_vrFilter = NULL;
+
+#ifdef LDAP_CONTROL_PAGEDRESULTS
 	op->o_pagedresults_state = conn->c_pagedresults_state;
+#endif
+
+	op->o_res_ber = NULL;
 
 #ifdef LDAP_CONNECTIONLESS
-	op->o_peeraddr = peeraddr;
-	if (cdn) {
-	    ber_str2bv( cdn, 0, 1, &op->o_dn );
-	    op->o_protocol = LDAP_VERSION2;
-	}
-#endif
-	if ( conn->c_conn_state == SLAP_C_BINDING
-		|| conn->c_conn_state == SLAP_C_CLOSING )
-	{
+	if (conn->c_is_udp) {
+		if ( cdn ) {
+		    ber_str2bv( cdn, 0, 1, &op->o_dn );
+		    op->o_protocol = LDAP_VERSION2;
+		}
+		op->o_res_ber = ber_alloc_t( LBER_USE_DER );
+		if (op->o_res_ber == NULL) return 1;
+
+		rc = ber_write( op->o_res_ber, (char *)&peeraddr,
+			sizeof(struct sockaddr), 0 );
+
+		if (rc != sizeof(struct sockaddr)) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-			   "connection_input: conn %lu  deferring operation\n",
-			   conn->c_connid ));
+			LDAP_LOG( CONNECTION, INFO, 
+				"connection_input: conn %lu  ber_write failed\n",
+				conn->c_connid, 0, 0 );
+#else
+			Debug( LDAP_DEBUG_ANY, "ber_write failed\n", 0, 0, 0 );
+#endif
+			return 1;
+		}
+
+		if (op->o_protocol == LDAP_VERSION2) {
+			rc = ber_printf(op->o_res_ber, "{is{" /*}}*/, op->o_msgid, "");
+			if (rc == -1) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( CONNECTION, INFO, 
+					"connection_input: conn %lu  put outer sequence failed\n",
+					conn->c_connid, 0, 0 );
+#else
+				Debug( LDAP_DEBUG_ANY, "ber_write failed\n", 0, 0, 0 );
+#endif
+				return rc;
+			}
+		}
+	}
+#endif /* LDAP_CONNECTIONLESS */
+
+	rc = 0;
+
+	/* Don't process requests when the conn is in the middle of a
+	 * Bind, or if it's closing. Also, don't let any single conn
+	 * use up all the available threads, and don't execute if we're
+	 * currently blocked on output. And don't execute if there are
+	 * already pending ops, let them go first.
+	 *
+	 * But always allow Abandon through; it won't cost much.
+	 */
+	if ( tag != LDAP_REQ_ABANDON && (conn->c_conn_state == SLAP_C_BINDING
+		|| conn->c_conn_state == SLAP_C_CLOSING
+		|| conn->c_n_ops_executing >= connection_pool_max/2
+		|| conn->c_n_ops_pending
+		|| conn->c_writewaiter))
+	{
+		int max = conn->c_dn.bv_len ? slap_conn_max_pending_auth
+			 : slap_conn_max_pending;
+#ifdef NEW_LOGGING
+		LDAP_LOG( CONNECTION, INFO, 
+			"connection_input: conn %lu  deferring operation\n",
+			conn->c_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "deferring operation\n", 0, 0, 0 );
 #endif
 		conn->c_n_ops_pending++;
 		LDAP_STAILQ_INSERT_TAIL( &conn->c_pending_ops, op, o_next );
-
+		if ( conn->c_n_ops_pending > max ) {
+			rc = -1;
+		} else {
+			rc = 1;
+		}
 	} else {
 		conn->c_n_ops_executing++;
-		connection_op_activate( conn, op );
+		connection_op_activate( op );
 	}
 
 #ifdef NO_THREADS
@@ -1446,7 +1566,7 @@ connection_input(
 #endif
 	assert( conn->c_struct_state == SLAP_C_USED );
 
-	return 0;
+	return rc;
 }
 
 static int
@@ -1464,9 +1584,9 @@ connection_resched( Connection *conn )
 
 		if( rc ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "connection", LDAP_LEVEL_DETAIL1,
-				   "connection_resched: conn %lu  reaquiring locks.\n",
-				   conn->c_connid ));
+			LDAP_LOG( CONNECTION, DETAIL1, 
+				"connection_resched: conn %lu  reaquiring locks.\n",
+				conn->c_connid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"connection_resched: reaquiring locks conn=%lu sd=%d\n",
@@ -1484,9 +1604,9 @@ connection_resched( Connection *conn )
 
 		if( conn->c_conn_state != SLAP_C_CLOSING ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "connection", LDAP_LEVEL_INFO,
-				   "connection_resched: conn %lu  closed by other thread.\n",
-				   conn->c_connid ));
+			LDAP_LOG( CONNECTION, INFO, 
+				"connection_resched: conn %lu  closed by other thread.\n",
+				conn->c_connid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE, "connection_resched: "
 				"closed by other thread conn=%lu sd=%d\n",
@@ -1494,9 +1614,9 @@ connection_resched( Connection *conn )
 #endif
 		} else {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "connection", LDAP_LEVEL_DETAIL1,
-				   "connection_resched: conn %lu  attempting closing.\n",
-				   conn->c_connid ));
+			LDAP_LOG( CONNECTION, DETAIL1, 
+				"connection_resched: conn %lu  attempting closing.\n",
+				conn->c_connid, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE, "connection_resched: "
 				"attempting closing conn=%lu sd=%d\n",
@@ -1509,12 +1629,15 @@ connection_resched( Connection *conn )
 		return 0;
 	}
 
-	if( conn->c_conn_state != SLAP_C_ACTIVE ) {
+	if( conn->c_conn_state != SLAP_C_ACTIVE || conn->c_writewaiter ) {
 		/* other states need different handling */
 		return 0;
 	}
 
 	while ((op = LDAP_STAILQ_FIRST( &conn->c_pending_ops )) != NULL) {
+		if ( conn->c_n_ops_executing > connection_pool_max/2 ) {
+			break;
+		}
 		LDAP_STAILQ_REMOVE_HEAD( &conn->c_pending_ops, o_next );
 		LDAP_STAILQ_NEXT(op, o_next) = NULL;
 		/* pending operations should not be marked for abandonment */
@@ -1523,7 +1646,7 @@ connection_resched( Connection *conn )
 		conn->c_n_ops_pending--;
 		conn->c_n_ops_executing++;
 
-		connection_op_activate( conn, op );
+		connection_op_activate( op );
 
 		if ( conn->c_conn_state == SLAP_C_BINDING ) {
 			break;
@@ -1532,46 +1655,44 @@ connection_resched( Connection *conn )
 	return 0;
 }
 
-static int connection_op_activate( Connection *conn, Operation *op )
+static int connection_op_activate( Operation *op )
 {
-	struct co_arg *arg;
 	int status;
 	ber_tag_t tag = op->o_tag;
 
 	if(tag == LDAP_REQ_BIND) {
-		conn->c_conn_state = SLAP_C_BINDING;
+		op->o_conn->c_conn_state = SLAP_C_BINDING;
 	}
 
-	arg = (struct co_arg *) ch_malloc( sizeof(struct co_arg) );
-	arg->co_conn = conn;
-	arg->co_op = op;
-
-	if (!arg->co_op->o_dn.bv_len) {
-	    arg->co_op->o_authz = conn->c_authz;
-	    arg->co_op->o_dn.bv_val = ch_strdup( conn->c_dn.bv_val ?
-	    	conn->c_dn.bv_val : "" );
-	    arg->co_op->o_ndn.bv_val = ch_strdup( conn->c_ndn.bv_val ?
-	    	conn->c_ndn.bv_val : "" );
+	if (!op->o_dn.bv_len) {
+	    op->o_authz = op->o_conn->c_authz;
+	    ber_dupbv( &op->o_dn, &op->o_conn->c_dn );
+	    ber_dupbv( &op->o_ndn, &op->o_conn->c_ndn );
 	}
-	arg->co_op->o_authtype = conn->c_authtype;
-	ber_dupbv( &arg->co_op->o_authmech, &conn->c_authmech );
+	op->o_authtype = op->o_conn->c_authtype;
+	ber_dupbv( &op->o_authmech, &op->o_conn->c_authmech );
 	
-	if (!arg->co_op->o_protocol) {
-	    arg->co_op->o_protocol = conn->c_protocol
-		? conn->c_protocol : LDAP_VERSION3;
+	if (!op->o_protocol) {
+	    op->o_protocol = op->o_conn->c_protocol
+		? op->o_conn->c_protocol : LDAP_VERSION3;
+	}
+	if (op->o_conn->c_conn_state == SLAP_C_INACTIVE
+		&& op->o_protocol > LDAP_VERSION2) {
+		op->o_conn->c_conn_state = SLAP_C_ACTIVE;
 	}
-	arg->co_op->o_connid = conn->c_connid;
 
-	LDAP_STAILQ_INSERT_TAIL( &conn->c_ops, arg->co_op, o_next );
+	op->o_connid = op->o_conn->c_connid;
+
+	LDAP_STAILQ_INSERT_TAIL( &op->o_conn->c_ops, op, o_next );
 
 	status = ldap_pvt_thread_pool_submit( &connection_pool,
-		connection_operation, (void *) arg );
+		connection_operation, (void *) op );
 
 	if ( status != 0 ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connection_op_activate: conn %lu	 thread pool submit failed.\n",
-			   conn->c_connid ));
+		LDAP_LOG( CONNECTION, ERR, 
+			"connection_op_activate: conn %lu	 thread pool submit failed.\n",
+			op->o_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		"ldap_pvt_thread_pool_submit failed (%d)\n", status, 0, 0 );
@@ -1585,6 +1706,7 @@ static int connection_op_activate( Connection *conn, Operation *op )
 int connection_write(ber_socket_t s)
 {
 	Connection *c;
+
 	assert( connections != NULL );
 
 	ldap_pvt_thread_mutex_lock( &connections_mutex );
@@ -1595,8 +1717,8 @@ int connection_write(ber_socket_t s)
 
 	if( c == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "connection", LDAP_LEVEL_ERR,
-			   "connection_write: sock %ld  no connection!\n",(long)s));
+		LDAP_LOG( CONNECTION, ERR, 
+			"connection_write: sock %ld  no connection!\n", (long)s, 0, 0);
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"connection_write(%ld): no connection!\n",
@@ -1610,9 +1732,8 @@ int connection_write(ber_socket_t s)
 	c->c_n_write++;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "connection", LDAP_LEVEL_DETAIL1,
-		   "connection_write conn %lu  waking output.\n",
-		   c->c_connid ));
+	LDAP_LOG( CONNECTION, DETAIL1, 
+		"connection_write conn %lu  waking output.\n", c->c_connid, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 		"connection_write(%d): waking output for id=%lu\n",
