@@ -1,7 +1,7 @@
 /* str2filter.c - parse an rfc 1588 string filter */
-/* $OpenLDAP: pkg/ldap/servers/slapd/str2filter.c,v 1.24.2.1 2002/04/15 15:21:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/str2filter.c,v 1.34 2003/05/28 20:17:08 hyc Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -16,25 +16,24 @@
 #include "slap.h"
 #include <ldap_pvt.h>
 
+#if 0 /* unused */
 static char	*find_matching_paren( const char *s );
+#endif /* unused */
 static Filter	*str2list( const char *str, long unsigned int ftype);
 static Filter	*str2simple( const char *str);
 static int	str2subvals( const char *val, Filter *f);
 
 Filter *
-str2filter( const char *str )
+str2filter_x( Operation *op, const char *str )
 {
 	int rc;
 	Filter	*f = NULL;
-	BerElement *ber;
-	char berbuf[256];
-	struct berval *bv = NULL;
-	Connection conn;
+	char berbuf[LBER_ELEMENT_SIZEOF];
+	BerElement *ber = (BerElement *)berbuf;
 	const char *text = NULL;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "filter", LDAP_LEVEL_ENTRY,
-		"str2filter: \"%s\"\n", str ));
+	LDAP_LOG( FILTER, ENTRY,  "str2filter: \"%s\"\n", str, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_FILTER, "str2filter \"%s\"\n", str, 0, 0 );
 #endif
@@ -43,9 +42,9 @@ str2filter( const char *str )
 		return NULL;
 	}
 
-	ber = ber_alloc_t( LBER_USE_DER );
-	if( ber == NULL ) {
-		return NULL;
+	ber_init2( ber, NULL, LBER_USE_DER );
+	if ( op->o_tmpmemctx ) {
+		ber_set_option( ber, LBER_OPT_BER_MEMCTX, &op->o_tmpmemctx );
 	}
 
 	rc = ldap_pvt_put_filter( ber, str );
@@ -53,25 +52,23 @@ str2filter( const char *str )
 		goto done;
 	}
 
-	rc = ber_flatten( ber, &bv );
-	if( rc < 0 ) {
-		goto done;
-	}
+	ber_reset( ber, 1 );
 
-	ber_free( ber, 0 );
-
-	ber = (BerElement *)berbuf;
-	ber_init2( ber, bv, 0 );
-
-	conn.c_connid = 0;
-
-	rc = get_filter( &conn, ber, &f, &text );
-	if( rc ) {
-		goto done;
-	}
+	rc = get_filter( op, ber, &f, &text );
 
 done:
-	ber_bvfree( bv );
+	ber_free_buf( ber );
 
 	return f;
 }
+
+Filter *
+str2filter( const char *str )
+{
+	Operation op = {0};
+
+	op.o_tmpmemctx = NULL;
+	op.o_tmpmfuncs = &ch_mfuncs;
+
+	return str2filter_x( &op, str );
+}
