@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/tools/slapcommon.h,v 1.3.2.3 2000/06/13 17:57:41 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/tools/slapcommon.h,v 1.13.2.1 2003/09/18 15:43:40 kurt Exp $ */
 /*
- * Copyright 1998-2000 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /* slapcommon.h - common definitions for the slap tools */
@@ -18,14 +18,22 @@ enum slaptool {
 	SLAPTEST	/* database testing tool */
 };
 
+#define SLAP_TOOL_CTXCSN_NONE	0
+#define SLAP_TOOL_CTXCSN_ENTRY	1
+#define SLAP_TOOL_CTXCSN_BATCH	2
 
 extern	char	*progname;
 extern	char	*conffile;
 extern	Backend *be;
 extern	int		appendmode;
 extern	int		verbose;
-extern	int		noschemacheck;
+extern	int		update_ctxcsn;
+extern	int		retrieve_ctxcsn;
+extern	int		retrieve_synccookie;
 extern	int		continuemode;
+extern	int		nosubordinates;
+extern	int		dryrun;
+extern	struct berval	sub_ndn;
 
 extern	char	*ldiffile;
 extern	FILE	*ldiffp;
