@@ -1,7 +1,7 @@
 /* unbind.c - decode an ldap unbind operation and pass it to a backend db */
-/* $OpenLDAP: pkg/ldap/servers/slapd/unbind.c,v 1.14.2.1 2002/04/15 15:21:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/unbind.c,v 1.19 2003/04/15 18:40:02 julius Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -26,21 +26,16 @@
 
 #include "slap.h"
 
-
 int
-do_unbind(
-    Connection	*conn,
-    Operation	*op
-)
+do_unbind( Operation *op, SlapReply *rs )
 {
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_ENTRY,
-		   "do_unbind: conn %d\n", conn ? conn->c_connid : -1 ));
+	LDAP_LOG( OPERATION, ENTRY, 
+		"do_unbind: conn %d\n", op->o_connid ? op->o_connid : -1, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "do_unbind\n", 0, 0, 0 );
 #endif
 
-
 	/*
 	 * Parse the unbind request.  It looks like this:
 	 *
@@ -51,7 +46,7 @@ do_unbind(
 	    op->o_opid, 0, 0, 0 );
 
 	/* pass the unbind to all backends */
-	backend_unbind( conn, op );
+	backend_unbind( op, rs );
 
 	return 0;
 }
