@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/config.c,v 1.25.2.1 2002/04/18 15:20:03 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slurpd/config.c,v 1.30.2.1 2003/09/18 15:43:40 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -34,19 +34,22 @@
 #include "slurp.h"
 #include "globals.h"
 
-#define MAXARGS	500
+#define ARGS_STEP	512
 
 /* Forward declarations */
 static void	add_replica LDAP_P(( char **, int ));
 static int	parse_replica_line LDAP_P(( char **, int, Ri *));
-static void	parse_line LDAP_P(( char *, int *, char ** ));
+static void	parse_line LDAP_P(( char * ));
 static char	*getline LDAP_P(( FILE * ));
 static char	*strtok_quote LDAP_P(( char *, char * ));
 
+int	cargc = 0, cargv_size = 0;
+char	**cargv;
 /* current config file line # */
 static int	lineno;
 
-
+char *slurpd_pid_file = NULL;
+char *slurpd_args_file = NULL;
 
 /*
  * Read the slapd config file, looking only for config options we're
@@ -60,12 +63,14 @@ slurpd_read_config(
 {
     FILE	*fp;
     char	*line;
-    int		cargc;
-    char	*cargv[MAXARGS];
+
+	cargv = ch_calloc( ARGS_STEP + 1, sizeof(*cargv) );
+	cargv_size = ARGS_STEP + 1;
 
 #ifdef NEW_LOGGING
-    LDAP_LOG (( "config", LDAP_LEVEL_ARGS, 
-	"slurpd_read_config: Config: opening config file \"%s\"\n", fname ));
+    LDAP_LOG ( CONFIG, ARGS, 
+		"slurpd_read_config: Config: opening config file \"%s\"\n", 
+		fname, 0, 0 );
 #else
     Debug( LDAP_DEBUG_CONFIG, "Config: opening config file \"%s\"\n",
 	    fname, 0, 0 );
@@ -84,13 +89,13 @@ slurpd_read_config(
 	}
 
 #ifdef NEW_LOGGING
-    LDAP_LOG (( "config", LDAP_LEVEL_DETAIL1, 
-		"slurpd_read_config: Config: (%s)\n", line ));
+    LDAP_LOG ( CONFIG, DETAIL1, 
+		"slurpd_read_config: Config: (%s)\n", line, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_CONFIG, "Config: (%s)\n", line, 0, 0 );
 #endif
 
-	parse_line( line, &cargc, cargv );
+	parse_line( line );
 
 	if ( cargc < 1 ) {
 	    fprintf( stderr, "line %d: bad config line (ignored)\n", lineno );
@@ -129,9 +134,9 @@ slurpd_read_config(
 
             if ( cargc < 2 ) {
 #ifdef NEW_LOGGING
-                LDAP_LOG(( "config", LDAP_LEVEL_CRIT,
+                LDAP_LOG( CONFIG, CRIT,
                         "%s: line %d: missing filename in \"include "
-                        "<filename>\" line.\n", fname, lineno ));
+                        "<filename>\" line.\n", fname, lineno , 0 );
 #else
                 Debug( LDAP_DEBUG_ANY,
         "%s: line %d: missing filename in \"include <filename>\" line\n",
@@ -149,13 +154,48 @@ slurpd_read_config(
 		
 	    free( savefname );
 	    lineno = savelineno - 1;
+
+	} else if ( strcasecmp( cargv[0], "replica-pidfile" ) == 0 ) {
+		if ( cargc < 2 ) {
+#ifdef NEW_LOGGING
+			LDAP_LOG( CONFIG, CRIT, 
+				"%s: line %d missing file name in \"replica-pidfile <file>\" "
+				"line.\n", fname, lineno, 0 );
+#else
+			Debug( LDAP_DEBUG_ANY,
+	    "%s: line %d: missing file name in \"replica-pidfile <file>\" line\n",
+				fname, lineno, 0 );
+#endif
+
+			return( 1 );
+		}
+
+		slurpd_pid_file = ch_strdup( cargv[1] );
+
+	} else if ( strcasecmp( cargv[0], "replica-argsfile" ) == 0 ) {
+		if ( cargc < 2 ) {
+#ifdef NEW_LOGGING
+			LDAP_LOG( CONFIG, CRIT, 
+				   "%s: %d: missing file name in "
+				   "\"argsfile <file>\" line.\n",
+				   fname, lineno, 0 );
+#else
+			Debug( LDAP_DEBUG_ANY,
+	    "%s: line %d: missing file name in \"argsfile <file>\" line\n",
+			    fname, lineno, 0 );
+#endif
+
+			return( 1 );
+		}
+
+		slurpd_args_file = ch_strdup( cargv[1] );
 	}
     }
     fclose( fp );
 #ifdef NEW_LOGGING
-    LDAP_LOG (( "config", LDAP_LEVEL_RESULTS, 
+    LDAP_LOG ( CONFIG, RESULTS, 
 		"slurpd_read_config: Config: "
-		"** configuration file successfully read and parsed\n" ));
+		"** configuration file successfully read and parsed\n", 0, 0, 0 );
 #else
     Debug( LDAP_DEBUG_CONFIG,
 	    "Config: ** configuration file successfully read and parsed\n",
@@ -172,19 +212,30 @@ slurpd_read_config(
  */
 static void
 parse_line(
-    char	*line,
-    int		*argcp,
-    char	**argv
+    char	*line
 )
 {
     char *	token;
 
-    *argcp = 0;
+    cargc = 0;
     for ( token = strtok_quote( line, " \t" ); token != NULL;
-	token = strtok_quote( NULL, " \t" ) ) {
-	argv[(*argcp)++] = token;
+	token = strtok_quote( NULL, " \t" ) )
+    {
+        if ( cargc == cargv_size - 1 ) {
+	    char **tmp;
+            tmp = ch_realloc( cargv, (cargv_size + ARGS_STEP) *
+                               sizeof(*cargv) );
+	    if (tmp == NULL) {
+		cargc = 0;
+		return;
+	    }
+	    cargv = tmp;
+            cargv_size += ARGS_STEP;
+        }
+
+	cargv[cargc++] = token;
     }
-    argv[*argcp] = NULL;
+    cargv[cargc] = NULL;
 }
 
 
@@ -330,11 +381,11 @@ add_replica(
 	sglob->num_replicas--;
     } else {
 #ifdef NEW_LOGGING
-    LDAP_LOG (( "config", LDAP_LEVEL_RESULTS, 
+    LDAP_LOG ( CONFIG, RESULTS, 
 		"add_replica: Config: ** successfully added replica \"%s%d\"\n", 
 		sglob->replicas[ nr - 1 ]->ri_hostname == NULL ?
 		"(null)" : sglob->replicas[ nr - 1 ]->ri_hostname,
-		sglob->replicas[ nr - 1 ]->ri_port, 0 ));
+		sglob->replicas[ nr - 1 ]->ri_port, 0 );
 #else
 	Debug( LDAP_DEBUG_CONFIG,
 		"Config: ** successfully added replica \"%s:%d\"\n",
@@ -391,9 +442,16 @@ parse_replica_line(
     int		gots = 0;
     int		i;
     char	*hp, *val;
+    LDAPURLDesc *ludp;
 
     for ( i = 1; i < cargc; i++ ) {
 	if ( !strncasecmp( cargv[ i ], HOSTSTR, sizeof( HOSTSTR ) - 1 ) ) {
+		if ( gots & GOT_HOST ) {
+			fprintf( stderr, "Error: Malformed \"replica\" line in slapd config " );
+			fprintf( stderr, "file, too many host or uri names specified, line %d\n",
+				lineno );
+			return -1;
+		}	
 	    val = cargv[ i ] + sizeof( HOSTSTR ); /* '\0' string terminator accounts for '=' */
 	    if (( hp = strchr( val, ':' )) != NULL ) {
 		*hp = '\0';
@@ -405,15 +463,46 @@ parse_replica_line(
 	    }
 	    ri->ri_hostname = strdup( val );
 	    gots |= GOT_HOST;
+	} else if ( !strncasecmp( cargv[ i ], URISTR, sizeof( URISTR ) - 1 ) ) {
+		if ( gots & GOT_HOST ) {
+			fprintf( stderr, "Error: Malformed \"replica\" line in slapd config " );
+			fprintf( stderr, "file, too many host or uri names specified, line %d\n",
+				lineno );
+			return -1;
+		}		
+		if ( ldap_url_parse( cargv[ i ] + sizeof( URISTR ), &ludp ) != LDAP_SUCCESS ) {
+			fprintf( stderr, "Error: Malformed \"replica\" line in slapd config " );
+			fprintf( stderr, "file, bad uri format specified, line %d\n",
+				lineno );
+			return -1;
+		}
+		if (ludp->lud_host == NULL) {
+			fprintf( stderr, "Error: Malformed \"replica\" line in slapd config " );
+			fprintf( stderr, "file, missing uri hostname, line %d\n",
+				lineno );
+			return -1;
+		}
+		ri->ri_hostname = strdup ( ludp->lud_host );
+		ri->ri_port = ludp->lud_port;
+		ri->ri_uri = strdup ( cargv[ i ] + sizeof( URISTR ) );		
+		ldap_free_urldesc( ludp );				
+	    gots |= GOT_HOST;
 	} else if ( !strncasecmp( cargv[ i ], 
 			ATTRSTR, sizeof( ATTRSTR ) - 1 ) ) {
 	    /* ignore it */ ;
 	} else if ( !strncasecmp( cargv[ i ], 
 			SUFFIXSTR, sizeof( SUFFIXSTR ) - 1 ) ) {
 	    /* ignore it */ ;
+	} else if ( !strncasecmp( cargv[i], STARTTLSSTR, sizeof(STARTTLSSTR)-1 )) {
+	    val = cargv[ i ] + sizeof( STARTTLSSTR );
+		if( !strcasecmp( val, CRITICALSTR ) ) {
+			ri->ri_tls = TLS_CRITICAL;
+		} else {
+			ri->ri_tls = TLS_ON;
+		}
 	} else if ( !strncasecmp( cargv[ i ], TLSSTR, sizeof( TLSSTR ) - 1 ) ) {
 	    val = cargv[ i ] + sizeof( TLSSTR );
-		if( !strcasecmp( val, TLSCRITICALSTR ) ) {
+		if( !strcasecmp( val, CRITICALSTR ) ) {
 			ri->ri_tls = TLS_CRITICAL;
 		} else {
 			ri->ri_tls = TLS_ON;
