@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/ri.c,v 1.21.2.2 2002/04/18 15:20:03 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slurpd/ri.c,v 1.26.2.1 2003/09/18 15:43:40 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -53,11 +53,13 @@ Ri_process(
     char	*errmsg;
 
     (void) SIGNAL( LDAP_SIGUSR1, do_nothing );
+#ifdef SIGPIPE
     (void) SIGNAL( SIGPIPE, SIG_IGN );
+#endif
     if ( ri == NULL ) {
 #ifdef NEW_LOGGING
-	LDAP_LOG (( "ri", LDAP_LEVEL_ERR, "Ri_process: "
-		"Error: ri == NULL!\n" ));
+	LDAP_LOG ( SLURPD, ERR, "Ri_process: "
+		"Error: ri == NULL!\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY, "Error: Ri_process: ri == NULL!\n", 0, 0, 0 );
 #endif
@@ -91,9 +93,9 @@ Ri_process(
 	    if ( !ismine( ri, re )) {
 		/* The Re doesn't list my host:port */
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ri", LDAP_LEVEL_DETAIL1, "Ri_process: "
+		LDAP_LOG ( SLURPD, DETAIL1, "Ri_process: "
 			"Replica %s:%d, skip repl record for %s (not mine)\n",
-			ri->ri_hostname, ri->ri_port, re->re_dn ));
+			ri->ri_hostname, ri->ri_port, re->re_dn );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"Replica %s:%d, skip repl record for %s (not mine)\n",
@@ -102,9 +104,9 @@ Ri_process(
 	    } else if ( !isnew( ri, re )) {
 		/* This Re is older than my saved status information */
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ri", LDAP_LEVEL_DETAIL1, "Ri_process: "
+		LDAP_LOG ( SLURPD, DETAIL1, "Ri_process: "
 			"Replica %s:%d, skip repl record for %s (old)\n",
-			ri->ri_hostname, ri->ri_port, re->re_dn ));
+			ri->ri_hostname, ri->ri_port, re->re_dn );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"Replica %s:%d, skip repl record for %s (old)\n",
@@ -116,9 +118,9 @@ Ri_process(
 		case DO_LDAP_ERR_RETRYABLE:
 		    ldap_pvt_thread_sleep( RETRY_SLEEP_TIME );
 #ifdef NEW_LOGGING
-			LDAP_LOG (( "ri", LDAP_LEVEL_DETAIL1, "Ri_process: "
+			LDAP_LOG ( SLURPD, DETAIL1, "Ri_process: "
 				"Retrying operation for DN %s on replica %s:%d\n",
-			    re->re_dn, ri->ri_hostname, ri->ri_port ));
+			    re->re_dn, ri->ri_hostname, ri->ri_port );
 #else
 		    Debug( LDAP_DEBUG_ANY,
 			    "Retrying operation for DN %s on replica %s:%d\n",
@@ -146,8 +148,8 @@ Ri_process(
 	    }
 	} else {
 #ifdef NEW_LOGGING
-		LDAP_LOG (( "ri", LDAP_LEVEL_ERR, "Ri_process: "
-			"Error: re is null in Ri_process\n" ));
+		LDAP_LOG ( SLURPD, ERR, "Ri_process: "
+			"Error: re is null in Ri_process\n", 0, 0, 0 );
 #else
 	    Debug( LDAP_DEBUG_ANY, "Error: re is null in Ri_process\n",
 		    0, 0, 0 );
@@ -210,6 +212,7 @@ Ri_init(
 
     /* Initialize private data */
     (*ri)->ri_hostname = NULL;
+    (*ri)->ri_uri = NULL;
     (*ri)->ri_ldp = NULL;
     (*ri)->ri_bind_dn = NULL;
     (*ri)->ri_password = NULL;
