@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/slurp.h,v 1.27 2002/02/08 05:44:34 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slurpd/slurp.h,v 1.30.2.1 2003/09/18 15:43:40 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -20,7 +20,7 @@
 #ifndef _SLURPD_H_
 #define _SLURPD_H_
 
-#ifndef LDAP_SYSLOG
+#if !defined(HAVE_WINSOCK) && !defined(LDAP_SYSLOG)
 #define LDAP_SYSLOG 1
 #endif
 
@@ -42,6 +42,15 @@
 #include "ldap_defaults.h"
 #include "ldif.h"
 
+#ifdef HAVE_WINSOCK
+#define ftruncate(a,b) _chsize(a,b)
+#define truncate(a,b) _lclose( _lcreat(a, 0))
+#define S_IRGRP 0
+#define S_IWGRP 0
+#endif
+
+#undef SERVICE_NAME
+#define SERVICE_NAME	OPENLDAP_PACKAGE "-slurpd"
 
 /* Default directory for slurpd's private copy of replication logs */
 #define	DEFAULT_SLURPD_REPLICA_DIR	LDAP_RUNDIR LDAP_DIRSEP "openldap-slurp"
@@ -108,8 +117,10 @@
 #define	T_MODOPREPLACE		9
 #define	T_MODOPDELETESTR	"delete"
 #define	T_MODOPDELETE		10
+#define	T_MODOPINCREMENTSTR	"increment"
+#define	T_MODOPINCREMENT	11
 #define	T_MODSEPSTR		"-"
-#define	T_MODSEP		11
+#define	T_MODSEP		12
 
 #define	T_NEWRDNSTR		"newrdn"
 #define	T_DELOLDRDNSTR	"deleteoldrdn"
@@ -119,6 +130,7 @@
 
 /* Config file keywords */
 #define	HOSTSTR			"host"
+#define	URISTR			"uri"
 #define	ATTRSTR			"attr"
 #define	SUFFIXSTR		"suffix"
 #define	BINDDNSTR		"binddn"
@@ -134,8 +146,9 @@
 #define	SASLMECHSTR		"saslmech"
 #define	REALMSTR		"realm"
 #define	SECPROPSSTR		"secprops"
+#define STARTTLSSTR		"starttls"
 #define TLSSTR			"tls"
-#define TLSCRITICALSTR	"critical"
+#define CRITICALSTR		"critical"
 
 #define	REPLICA_SLEEP_TIME	( 10 )
 
@@ -204,6 +217,7 @@ struct ri {
     /* Private data */
     char	*ri_hostname;		/* canonical hostname of replica */
     int		ri_port;		/* port where slave slapd running */
+    char	*ri_uri;		/* e.g. "ldaps://ldap-1.example.com:636" */
     LDAP	*ri_ldp;		/* LDAP struct for this replica */
     int		ri_tls;			/* TLS: 0=no, 1=yes, 2=critical */
     int		ri_bind_method;		/* AUTH_SIMPLE or AUTH_KERBEROS */
